/*
 * Xelfi
 */

package xelfi.awt;

import java.awt.*;

/**
 * A class that produces ColorGrid component.
 *
 * @version 2.1 1 Mar 1997
 * @author  Petr Hamernik
 */
public class ColorGrid extends Canvas {

   /** When the setected color (foreground or background) change by pressing mouse button, 
    *  parent will get the event with this id. 
    */
   public final static int COL_GRID_CHANGE = 529600;

   /** Constant for changing the grid ordering: 1 column and 16 rows.
    * @see #gridOrdering
    */
   public final static int GO_1x16 = 1;
   
   /** Constant for changing the grid ordering: 2 columns and 8 rows.
    * @see #gridOrdering
    */
   public final static int GO_2x8  = 2;
   
   /** Constant for changing the grid ordering: 4 columns and 4 rows.
    * @see #gridOrdering
    */
   public final static int GO_4x4  = 3;
   
   /** Constant for changing the grid ordering: 8 columns and 2 rows.
    * @see #gridOrdering
    */
   public final static int GO_8x2  = 4;

   /** Constant for changing the grid ordering: 16 columns and 1 row.
    * @see #gridOrdering
    */
   public final static int GO_16x1 = 5;

   /** Current grid ordering. For setting new grid ordering use setGridOrdering method.
    * @see #setGridOrdering
    * @see #getGridOrdering
    * @see #GO_1x16
    * @see #GO_2x8
    * @see #GO_4x4
    * @see #GO_8x2
    * @see #GO_16x1
    */
   protected int gridOrdering;

   /** When the ColorGrid is created the gridOrdering is set to this value. 
    * Default is GO_4x4.
    * @see #GO_4x4
    * @see #gridOrdering
    */
   public final static int defaultGridOrdering = GO_4x4;

   /** Enable selection of foreground color. */
    protected boolean fgEnabled;
   
   /** Enable selection of background color. */
    protected boolean bgEnabled;
   
   /** Default index of foreground color */
   public final static int defaultfgCol = 11;

   /** Default index of background color */
   public final static int defaultbgCol = 0;

   /** Curent index of foreground color */
   protected int fgCol;

   /** Curent index of background color */
   protected int bgCol;
   
   /** Border color around the grid cells. By default it is set to defaultBorderColor.
    * @see #setBorderColor
    * @see #getBorderColor
    * @see #defaultBorderColor
    */
    protected Color marginColor = null;

   /** Default border color around the grid cells. Currently light gray with the R/G/B
    *  192/192/192
    * @see #setBorderColor
    * @see #getBorderColor
    * @see #marginColor
    */
    public static final Color defaultBorderColor = new Color(192, 192, 192);

   /** Width of border around the grid cells.
    * @see #defaultBorderWidth
    * @see #setBorderWidth
    * @see #getBorderWidth
    */     
    protected int margin = 0;
   
   /** Default width of 3D border around the grid cells. Currently 1.
    * @see #margin
    * @see #setBorderWidth
    * @see #getBorderWidth
    */
    public static final int defaultBorderWidth = 1;
   
   /** Preferred size of one cell */
   private static final int prefDim = 20;

   /** Minimum size of one cell */
   private static final int minDim = 12;

   /** Array of colors which are in the grid. These colors can be changed calling
    *  method setGridColor
    * @see #setGridColor
    * @see #getGridColor
    */
   protected Color colors[];

   /** If propagate is set to true all events are propagate and don't
    *  catch. This flag is used for xawt component during design time.
    */
   public boolean propagate = false;

   /** Create new color grid component with all default values. */
   public ColorGrid() {
       setDefaults();
   }

   private void setDefaults() {
       gridOrdering = defaultGridOrdering;
       marginColor = defaultBorderColor;
       margin = defaultBorderWidth;
       colors = new Color[16];
       stdColors();
       fgEnabled = true;
       bgEnabled = true;
       fgCol = defaultfgCol;
       bgCol = defaultbgCol;
   }
       
   /** Sets the new grid ordering.
    * @param go New value of gridOrdering. Allowed values are
    *      the GO_xxx constants.
    * @see #getGridOrdering
    * @see #GO_1x16
    * @see #GO_2x8
    * @see #GO_4x4
    * @see #GO_8x2
    * @see #GO_16x1
    * @exception ArrayIndexOutOfBoundsException If the index was invalid.
    */
   public void setGridOrdering(int go) throws ArrayIndexOutOfBoundsException {
       if ((go < 1)||(go > 5))
           throw new ArrayIndexOutOfBoundsException();
       gridOrdering =  go;
       repaint();
   }

   /** Gets the current value of gridOrdering.
    * @return Current value of gridOrdering.
    * @see #setGridOrdering
    * @see #GO_1x16
    * @see #GO_2x8
    * @see #GO_4x4
    * @see #GO_8x2
    * @see #GO_16x1
    */
   public int getGridOrdering() {
       return gridOrdering;
   }

   private void stdColors() {
       colors[0] = Color.black;
       colors[1] = Color.blue;
       colors[2] = Color.cyan;
       colors[3] = Color.darkGray;
       colors[4] = Color.gray;
       colors[5] = Color.green;
       colors[6] = Color.lightGray;
       colors[7] = Color.magenta;
       colors[8] = Color.orange;
       colors[9] = Color.pink;
       colors[10] = Color.red;
       colors[11] = Color.white;
       colors[12] = Color.yellow;
       colors[13] = new Color(150,0,0);
       colors[14] = new Color(0,150,0);
       colors[15] = new Color(0,0,150);
   }

   /** Sets new color at the explicit position in the grid.
    * @param index Index of the position in the grid.
    * @param col New value of color.
    * @exception ArrayIndexOutOfBoundsException If the index was invalid.
    */
   public void setGridColor(int index, Color col) throws ArrayIndexOutOfBoundsException {
       if ((index < 0)||(index > 15))
           throw new ArrayIndexOutOfBoundsException();
       colors[index] = col;
       repaint();
   }

   /** Gets the color at the explicit position in the grid.
    * @return Current value of the color at this position.
    * @exception ArrayIndexOutOfBoundsException If the index was invalid.
    */
   public Color getGridColor(int index) throws ArrayIndexOutOfBoundsException {
       if ((index < 0)||(index > 15))
           throw new ArrayIndexOutOfBoundsException();
       return colors[index];
   }

   /** Sets the default colors in the grid */
   public void setDefaultColors() {
       stdColors();
       repaint();
   }   

   /** Sets the selection of the foreground color.
    * @param index New index of the foreground color.
    * @exception ArrayIndexOutOfBoundsException If the index was invalid.
    * @see #fgCol
    * @see #getFg
    * @see #getFgColor
    */
   public void setFg(int index) throws ArrayIndexOutOfBoundsException {
       if ((index < 0)||(index > 15))
           throw new ArrayIndexOutOfBoundsException();
       if (fgCol == index)
           return;
       int old = fgCol;
       fgCol = index;
       repaintBox(fgCol);
       repaintBox(old);
   }

   /** Gets the index of currently selected foreground color.
    * @return Value of fgCol.
    * @see #fgCol
    * @see #getFgColor
    * @see #setFg
    */
   public int getFg() {
       return fgCol;
   }

   /** Gets the currently selected foreground color.
    * @return Selected foreground color.
    * @see #fgCol
    * @see #getFg
    * @see #setFg
    */
   public Color getFgColor() {
       return colors[fgCol];
   }

   /** Sets the selection of the background color.
    * @param index New index of the background color.
    * @exception ArrayIndexOutOfBoundsException If the index was invalid.
    * @see #bgCol
    * @see #getBg
    * @see #getBgColor
    */
   public void setBg(int index) throws ArrayIndexOutOfBoundsException {
       if ((index < 0)||(index > 15))
           throw new ArrayIndexOutOfBoundsException();
       if (bgCol == index)
           return;
       int old = bgCol;
       bgCol = index;
       repaintBox(index);
       repaintBox(old);
   }

   /** Gets the index of currently selected background color.
    * @return Value of bgCol.
    * @see #bgCol
    * @see #getBgColor
    * @see #setBg
    */
   public int getBg() {
       return bgCol;
   }

   /** Gets the currently selected background color.
    * @return Selected background color.
    * @see #bgCol
    * @see #getBg
    * @see #setBg
    */
   public Color getBgColor() {
       return colors[bgCol];
   }

   /** Enable or disable the selection of foreground color.
    * @param en if true selection of foreground color will be
    *      enabled, if false then disabled.
    * @see #fgIsEnable
    */
    public void fgEnable(boolean en) {
       fgEnabled = en;
       repaintBox(fgCol);
    }

   /** Gets the current value of fgEnabled.
   * @return true if selection of foreground is enabled otherwise
   *       false.
    * @see #fgEnable
   */
    public boolean fgIsEnable() {
        return fgEnabled;
    }

   /** Enable or disable the selection of background color.
    * @param en if true selection of background color will be
    *      enabled, if false then disabled.
    * @see #bgIsEnable
    */
    public void bgEnable(boolean en) {
       bgEnabled = en;
       repaintBox(bgCol);
    }

   /** Gets the current value of bgEnabled.
   * @return true if selection of background is enabled otherwise
   *       false.
    * @see #bgEnable
   */
    public boolean bgIsEnable() {
        return bgEnabled;
    }

   /** Sets the border color.
    * @param col New value of marginColor.
    * @see #marginColor
    * @see #defaultBorderColor
    * @see #getBorderColor
    */
    public void setBorderColor(Color col) {
        marginColor = col;
        repaint();
    }

   /** Gets the border color.
    * @return Current border color.
    * @see #marginColor
    * @see #defaultBorderColor
    * @see #setBorderColor
    */
    public Color getBorderColor() {
        return marginColor;
    }

   /** Sets border width around the component.
    * @param borderWidth New value of margin.
    * @see #margin
    * @see #getBorderWidth
    */
    public void setBorderWidth(int borderWidth) {
        margin = borderWidth;
        repaint();
    }

   /** Gets border width around the component.
    * @see #margin
    * @see #setBorderWidth
    */
    public int getBorderWidth() {
        return(margin);
    }

    public boolean mouseDown(Event event, int x, int y) {
       if (propagate)
           return false;
       Dimension d = size();
       Dimension gr = resolveGrid();
       int newSel = (gr.height*((gr.width * x)/d.width) + (gr.height * y)/d.height);
       if ((event.modifiers & Event.META_MASK) != 0)
           setBg(newSel);
       else
           setFg(newSel);
       event.id = COL_GRID_CHANGE;
       return false;
   }

	public void paint(Graphics g) {
		Dimension d = size();
		Dimension gr = resolveGrid();
		int col = 0;
		for (int i = 0; i < gr.width; i++) {
			for (int j = 0; j < gr.height; j++) {
				int x = (d.width * i) / gr.width;
				int y = (d.height * j) / gr.height;
				int w = (d.width * (i+1)) / gr.width - x - 1;
				int h = (d.height * (j+1)) / gr.height - y - 1;
				paintBox(g,x,y,w,h,col);
				paintText(g,x,y,w,h,col);
				col++;
			}
		}
	}

   private void repaintBox(int index) {
       Dimension d = size();
       Dimension gr = resolveGrid();
       int i = index / gr.height;
       int j = index - i*gr.height;
       int x = (d.width * i) / gr.width;
       int y = (d.height * j) / gr.height;
       int w = (d.width * (i+1)) / gr.width - x - 1;
       int h = (d.height * (j+1)) / gr.height - y - 1;
       Graphics g = getGraphics();
	   if (g == null)
		   return;
       paintBox(g, x, y, w, h, index);
       paintText(g, x, y, w, h, index);
   }

	private void paintBox(Graphics g, int x, int y, int w, int h, int index) {
		g.setColor(marginColor);
		for (int i = 0; i < margin; i++) {
		g.draw3DRect(x, y, w, h, false);
			x++;
			y++;
			w -= 2;
			h -= 2;
		}
		g.setColor(colors[index]);
		g.fillRect(x,y,w+1,h+1);
	}

	private void paintText(Graphics g, int x, int y, int w, int h, int index) {
       if ((fgEnabled)&&(bgEnabled)&&(bgCol == index)&&(fgCol == index))
            paintSelect(g,x,y,w,h,"FB",index);
       else {
          if ((index == fgCol)&&(fgEnabled))
               paintSelect(g,x,y,w,h,"F",index);
          if ((index == bgCol)&&(bgEnabled))
               paintSelect(g,x,y,w,h,"B",index);
       }
	}

	private void paintSelect(Graphics g, int x, int y, int w, int h, String type, int index) {
       int rc = 0;
       int gc = 0;
       int bc = 0;
       if (colors[index].getRed() < 127)
           rc = 255;
       if (colors[index].getGreen() < 127)
           gc = 255;
       if (colors[index].getBlue() < 127)
           bc = 255;
       g.setColor(new Color(rc,gc,bc));
       Font font = getFont();
       FontMetrics f = Toolkit.getDefaultToolkit().getFontMetrics(
               font==null ? new Font("Helvetica", Font.PLAIN, 12) : font);
       int w1 = f.stringWidth(type);
       int h1 = f.getHeight();
       g.drawString(type, x + (w-w1)/2, y + (h+h1)/2);
   }

    public Dimension minimumSize() {
        return countSize(minDim);
    }

    public Dimension preferredSize() {
        return countSize(prefDim);
    }

   private Dimension resolveGrid() {
       int x = 0;
       int y = 0;
       switch (gridOrdering) {
           case GO_1x16:   x = 1; y = 16; break;
           case GO_2x8:    x = 2; y = 8; break;
           case GO_4x4:    x = 4; y = 4; break;
           case GO_8x2:    x = 8; y = 2; break;
           case GO_16x1:   x = 16; y = 1; break;
       }
       return new Dimension(x,y);
   }

   private Dimension countSize(int dim) {
       Dimension d = resolveGrid();
       d.width = d.width * (dim + 2 * margin);
       d.height = d.height * (dim + 2 * margin);
       return d;
   }
}

