package xelfi.awt;

import java.awt.*;
import java.util.Vector;

import xelfi.util.*;     // for XelfiPath and Debug

/**
 * TreeView shows tree structure of objects which implements
 * interface XelfiTreeViewElement on the screen.
 *
 * @version 1.01, 19 Nov 1996
 * @author Dafe
   Last change:  DF   17 Mar 97   12:24 pm
*/
public class XFTreeView {

   private static final int      X_INDENT = 10;
   private static final int      X_IMAGE_INDENT = 2;
   private static final int      Y_INDENT = 4;
   private static final int      TREE_INDENT = 15;
   private static final int      H_INDENT = 5;

   private XFTree                tree;

   // list of elements of the tree which are not hidden by pulling
   private Vector                treeList;
   // list of selected elements
   private Vector                  selectedItems;
   // helper variable for proper painting
   private int                     globalY;
   // array of elements' bounds - need for implementing
   // getElementFromPoint and getPullIconFromPoint methods
   private Vector                elementBounds;
   private Vector                imagesBounds;
   // holds preferred Size
   private Dimension               myPrefSize;
   // image tracker for their loading,etc..
   private MediaTracker            tracker;
   private boolean                   imagesLoaded = false;
   // node images
   private   Image                   folderImage, nodeImage,
                                 openFolderImage;
  private int                    imageIndex;
   // parent component
   private Component               parentComp;
   // multiple selection flag
   private boolean                 multipleSelections;
   // current visible rectangle (used fro faster drawing)
   private Rectangle               visibleRect;
   // move of visible window coordinates
   private int                   moveX;
   private int                   moveY;


   public XFTreeView (Object rootData, Component parentComp) {
     this.parentComp = parentComp;
     tree = new XFTree(rootData);
     treeList = new Vector(30, 10);
     selectedItems = new Vector(10, 10);
     elementBounds = new Vector(30, 10);
     imagesBounds = new Vector(30, 10);
     multipleSelections = false;
     // create tracker, adds tree node images
     tracker = new MediaTracker(this.parentComp);
     addImages();
     // wait until images are loaded
     waitForImages();
     // initialize list and element bounds
     refreshList();
     // select root element
     selectNode(0);
   }

   public XFTreeView (Component parentComp) {
     this (null, parentComp);
   }

   public Dimension preferredSize () {
     Debug.debug(Debug.INSPECTOR, "pref height is "+myPrefSize.height);
     return new Dimension(myPrefSize);
   }

  public Dimension minimumSize () {
    return new Dimension(50, 30);
  }

   /**
    * Access method.
   */
   public XFTree getTree () {
     return tree;
   }


  /**
   * Sets new tree. (changes content)
  */
  public void setTree (XFTree newTree) {
		Debug.debug(Debug.INSPECTOR, "Jsem v settree");
    tree = newTree;
    deselectAll();
    refreshList();
    // select root element
    selectNode(0);
   }

   /**
    * Inserts new view element after currently selected.<br>
    * Inserts after first selected for multiple selections.<br>
    * Does nothing when no item is selected.
   */
  public void insertAfter (XelfiTreeViewElement element) {
    int current = getSelectedIndex();
    if (current==-1) return;
    XFTreeNode previousCurNode = (XFTreeNode)treeList.elementAt(current);
    tree.insertAfter((Object)element, previousCurNode);
    // jump to the next element on this level
    // be careful if current was pulled up
    int delta = ((XelfiTreeViewElement)previousCurNode.getData()).
                isPulled() ? previousCurNode.getAllChildCount()+1 : 1;
    refreshList();
    if (multipleSelections)
      deselectAll();
    selectNode(current+delta);
   }

   /**
    * Inserts new view element into currently selected.<br>
    * Inserts after first selected for multiple selections.<br>
    * Inserts as very first item when no item is selected.
   */
   public void insertInto (XelfiTreeViewElement element) {
     int current = getSelectedIndex();
     if (current==-1) {
		   // no item is selected, insert as very first
	     tree.insertInto((Object)element,
                       tree.getRoot());
		 } else {
       tree.insertInto((Object)element,
                   (XFTreeNode)treeList.elementAt(current));
		 }
     refreshList();
     if (multipleSelections)
       deselectAll();
     selectNode(current+1);
   }

   /**
    * Inserts new view element after specified node.<br>
    * Does nothing when no such node is found.
   */
  public void insertAfter (XFTreeNode node, XelfiTreeViewElement element) {
     if (getIndex(node)==-1) return;
     tree.insertAfter((Object)element, node);
     refreshList();
   }

   /**
    * Inserts new view element into currently selected.<br>
    * Does nothing when no such node is found.
   */
   public void insertInto (XFTreeNode node, XelfiTreeViewElement element) {
     if (getIndex(node)==-1) return;
     tree.insertInto((Object)element, node);
     refreshList();
   }

   /**
    * Removes currently selected element.<br>
    * Removes first selected for multiple selections.<br>
    * Does nothing when no item is selected.
   */
   public void remove () {
     int current = getSelectedIndex();
     if (current==-1) return;
     tree.remove((XFTreeNode)treeList.elementAt(current));
     // and refresh element list and selection
     refreshList();
     refreshSelection();
     // if single selection mode, selects item next to deleted
     if (!multipleSelections)
       selectNode(current>=treeList.size() ? treeList.size()-1 : current);
   }

   /**
    * Pulls currently selected element down or up, if it's possible.<br>
    * Pulls first selected for multiple selections.<br>
    * Does nothing when no item is selected.
   */
   public void pull (boolean state) {
     int current = getSelectedIndex();
     if (current==-1) return;
     ((XelfiTreeViewElement)((XFTreeNode)treeList.elementAt(current)).
                             getData()).pull(state);
     refreshList();
   }

   /**
    * Toggles the pull state of currently selected item.<br>
    * Toggles first selected for multiple selections.<br>
    * Does nothing when no item is selected.
   */
   public void togglePull () {
     togglePull(getSelectedIndex());
   }

   /**
    * Toggles the pull state of specified item.
   */

   public void togglePull (int index) {
     if ((index<0) || (index>=treeList.size())) return;
     pull(((XelfiTreeViewElement)((XFTreeNode)treeList.elementAt(index)).
           getData()).isPulled() ? false : true);
   }


   /**
    * Paints whole tree.
   */
   public void paint (Graphics g, int x, int y, int width, int height) {
     if (!imagesLoaded)
       waitForImages();
//     elementBounds.removeAllElements();
//     imagesBounds.removeAllElements();
     globalY = y+getInitialIndent();
     visibleRect = new Rectangle(0, 0, width, height);
     // save current move values
     moveX = x;
     moveY = y;
		 // paint the tree for all children of root
		 XFTreeNode root = tree.getRoot();
		 for (int i=0; i<root.getChildCount(); i++) {
	     paintTree(g, root.getChild(i), x, globalY);
		 }
   }


   /**
    * Paint item specified by index.
   */
   public void paintItem (Graphics g, int index, int moveX, int moveY,
                          boolean clearBackground) {
     if ((index<0) || (index>=treeList.size()) || (g==null))
       return;
     Rectangle itemBounds = (Rectangle)elementBounds.elementAt(index);
     XelfiTreeViewElement item = getItem(index);
     if (clearBackground)
       g.clearRect(moveX+itemBounds.x, moveY+itemBounds.y,
                   itemBounds.width, itemBounds.height);
     if (isSelected(index)) {
       item.paintSelected(g, moveX+itemBounds.x,
                          moveY+itemBounds.y+itemBounds.height/2);
     } else {
       item.paint(g, moveX+itemBounds.x,
                  moveY+itemBounds.y+itemBounds.height/2);
     }
   }

   /**
    * Selects next element in tree (from top to down)<br>
    * ! Owner class is responsible for proper redrawing !
    * Returns false if there's no next element.
    * Selects next element to first selected
    * when multiple selections is on
   */
   public boolean next () {
     return next(getSelectedIndex());
   }

   /**
    * Selects element next to element specified by index.
   */
   public boolean next (int index) {
     if ((index==-1) || (index>=(treeList.size()-1))) return false;
     deselectAll();
     selectNode(index+1);
     return true;
   }

   /**
    * Selects previous element in tree (from top to down)
    * if it is possible.<br>
    * ! Owner class is responsible for proper redrawing !<br>
    * Returns false if there's no previous element.
   */
   public boolean previous () {
     return previous(getSelectedIndex());
   }

   /**
    * Selects element previous to element specified by index.
   */
   public boolean previous (int index) {
     if ((index==-1) || (index<=0)) return false;
     deselectAll();
     selectNode(index-1);
     return true;
   }

   /**
    * Deselects all selected items.
   */
   public void deselectAll () {
     selectedItems.removeAllElements();
   }

   /**
    * Returns currently selected node.<br>
    * Returns first selected for multiple selections.<br>
    * Returns null when no item is selected.
   */
   public XFTreeNode getCurNode () {
     int current = getSelectedIndex();
     if (current==-1) return null;
     return (XFTreeNode)treeList.elementAt(current);
   }

   /**
    * Returns currently selected item.<br>
    * Returns first selected for multiple selections.<br>
    * Returns null when no item is selected.
   */
   public XelfiTreeViewElement getCurItem () {
     int current = getSelectedIndex();
     if (current==-1) return null;
     return (XelfiTreeViewElement)
       (((XFTreeNode)treeList.elementAt(current)).getData());
   }

   /**
    * Returns index of currently selected item.
    * Returns -1 if no item is selected.
   */
   public int getCurIndex () {
     return getSelectedIndex();
   }

   /**
    * Returns node at index position (can return only elements
    * which are pulled down at this time, can not return elements
    * which are hidden because its parent is pulled up)<br>
    * Returns null on error.
   */
   public XFTreeNode getNode (int index) {
     if ((index<0) && (index>=treeList.size())) return null;
     return (XFTreeNode)treeList.elementAt(index);
   }

   /**
    * Returns item at index position.<br>
    * Returns null when index is out of bounds.
   */
   public XelfiTreeViewElement getItem (int index) {
     if ((index<0) && (index>=treeList.size())) return null;
     return (XelfiTreeViewElement)
             (((XFTreeNode)treeList.elementAt(index)).getData());
   }

   /**
    * Returns count of elements in tree.<br>
    * Doesn't count elements which are hidden by pulling.
   */
   public int getCount () {
     return treeList.size();
   }

   /**
    * Sets selection to node at index.<br>
    * Returns false if index position doesn't exist
    * or if item was already selected before.
   */
   public boolean selectNode (int index) {
     if ((index<0) || (index>=treeList.size())) return false;
     if (isSelected(index)) return false;
     if (!multipleSelections)
       deselectAll();
     selectedItems.addElement(treeList.elementAt(index));
     return true;
   }

   /**
    * Sets selection to node(node mustn't be hidden by pulling)<br>
    * Returns false if node can't be found
   */
   public boolean selectNode (XFTreeNode node) {
     return selectNode(treeList.indexOf(node));
   }

   /**
    * Sets selection to node. If the node is pulled up at the moment,
    * neccesary pulling is done.<br>
    * Returns false if node can't be found
   */
   public boolean selectAbsNode (XFTreeNode node) {
     if (node==null) return false;
     boolean result = selectNode(node);
     if (result==false) {
       for (XFTreeNode i=node.getParent(); i!=null; i=i.getParent()) {
         ((XelfiTreeViewElement)i.getData()).pull(true);
       }
       refreshList();
       // and select again...selectNode shouldn't fail here
       result = selectNode(node);
       if (result!=true) {
         Debug.debug(Debug.INSPECTOR, "!!! Something goes wrong in selectAbsNode...");
       }
     }
     return result;
   }

   /**
    * Returns node which contains specified point.<br>
    * Returns null if there's no node containing the point.
   */
   public XFTreeNode getNodeFromPoint (int x, int y) {
     for (int i=0; i<elementBounds.size(); i++) {
       if (((Rectangle)elementBounds.elementAt(i)).inside(x, y))
         return (XFTreeNode)treeList.elementAt(i);
     }
     return null;
   }

   /**
    * Returns node which pull icon contains specified point.<br>
    * Returns null if there's no pull icon containing the point.
   */
   public XFTreeNode getPullIconFromPoint (int x, int y) {
     for (int i=0; i<imagesBounds.size(); i++) {
       if (((Rectangle)imagesBounds.elementAt(i)).inside(x, y))
         return (XFTreeNode)treeList.elementAt(i);
     }
     return null;
   }

   /**
    * Returns bounds of element at specified index.
    * (indexes counted from 0 and only pulled down are counted)
   */
   public Rectangle getElementBounds (int index) {
     if ((index<0) || (index>=elementBounds.size())) return null;
     return (Rectangle)elementBounds.elementAt(index);
   }

   /**
    * Sets closed folder image.<br>
    * imageFile must be valid full path to the image file.
    * When image loading fails, returns false
   */
   public boolean setFolderImage (String imageFile) {
     folderImage = getImage(imageFile);
     return waitForImage(folderImage);
   }

   /**
    * Sets closed folder image.<br>
   * When image loading fails, returns false
   */
   public boolean setFolderImage (Image image) {
     folderImage = image;
     return waitForImage(folderImage);
   }

   /**
    * Sets open folder image.<br>
    * imageFile must be valid full path to the image file.
    * When image loading fails, returns false
   */
   public boolean setOpenFolderImage (String imageFile) {
     openFolderImage = getImage(imageFile);
     return waitForImage(folderImage);
   }

   /**
    * Sets open folder image.<br>
    * When image loading fails, returns false
   */
   public boolean setOpenFolderImage (Image image) {
     openFolderImage = image;
     return waitForImage(folderImage);
   }

   /**
    * Sets open folder image.<br>
    * imageFile must be valid full path to the image file.
    * When image loading fails, returns false
   */
   public boolean setNodeImage (String imageFile) {
     nodeImage = getImage(imageFile);
     return waitForImage(nodeImage);
   }

   /**
    * Sets open folder image.<br>
    * When image loading fails, returns false
   */
   public boolean setNodeImage (Image image) {
     nodeImage = image;
     return waitForImage(nodeImage);
   }

   /**
    * Returns true if this list allows multiple selections.
   */
   public boolean allowsMultipleSelections () {
     return multipleSelections;
   }

   /**
    * Returns the number of selected items in the list.
   */
   public int countSelectedItems () {
     return selectedItems.size();
   }

   /**
    * Delete an item at index (only pulled down nodes) 
    * from the list.<br>
    * Returns false if specified item couldn't be deleted;
    * true otherwise.
   */
  public boolean delItem (int index) {
     if (index<0 || index>=getCount()) return false;
     Object item = treeList.elementAt(index);
     tree.remove((XFTreeNode)item);
     refreshList();
     refreshSelection();
     return true;
   }

   /**
    * Deselects the item at the specified index.
    * Returns false if specified item hasn't been selected;
    * true otherwise.
   */
   public boolean deselect (int index) {
     if (index<0 || index>=getCount()) return false;
     Object item = treeList.elementAt(index);
     return selectedItems.removeElement(treeList.elementAt(index));
   }

   /**
    * Get the selected item on the list or -1 if no item is selected.
    * If multiple selections is set, returns index of 
    * first selected item.
   */
   public int getSelectedIndex () {
     if (selectedItems.size()<=0) return -1;
     return treeList.indexOf(selectedItems.elementAt(0));
   }

   /**
    * Returns the selected indexes on the list.
   */
   public int[] getSelectedIndexes () {
     int selectedCount = countSelectedItems();
     int[] selectedIndexes = new int[selectedCount];
     
     for (int i=0; i<selectedCount; i++) {
       selectedIndexes[i] = treeList.indexOf(selectedItems.elementAt(i));
     }
     return selectedIndexes;
   }

   /**
    * Returns the selected items on the list.
   */
   public XelfiTreeViewElement[] getSelectedItems () {
     int selectedCount = countSelectedItems();
     XelfiTreeViewElement[] selItems =
           new   XelfiTreeViewElement[selectedCount];
     for (int i=0; i<selectedCount; i++) {
       selItems[i] = (XelfiTreeViewElement)
                     (((XFTreeNode)(selectedItems.elementAt(i))).getData());
     }
     return selItems;
   }

   /**
    * Returns true if the item at the specified index has been selected;
    * false otherwise.
   */
   public boolean isSelected (int index) {
     if (index<0 || index>=getCount()) return false;
     return selectedItems.indexOf(treeList.elementAt(index))!=-1;
   }

   /**
    * Conditionally selects or unselects item at the specified index.
   */
   public void select (int index, boolean selectItem) {
     if (selectItem)
       selectNode(index);
     else
       deselect(index);
   }

   /**
    * Enables/disables multiple selections for list.
   */
   public boolean setMultipleSelections (boolean multiple) {
     if (multipleSelections==multiple) return false;
     multipleSelections = multiple;
     deselectAll();
     return true;
   }

   /**
    * Refreshes content of treeview and element bounds.
    * Needed when preferred size of some element inthe tree is changed.
   */
   public void refresh () {
     refreshList();
   }

   /**
    * Returns index of node in tree. (only visible items)
    * Returns -1 when no such node exists.
   */
   public int getIndex (XFTreeNode node) {
     return treeList.indexOf(node);
   }

	/**
	 * Moves the source node into destination node
	*/
	public boolean moveItem (XFTreeNode destination,
												   XFTreeNode source, XFTreeNode after) {
		boolean result = tree.moveItem(destination, source, after);
		if (result) {
      refreshList();
      deselectAll();
			selectNode(source);
		}
		return result;
	}

   /**
    * Go recursively through the tree (method - search to the deep)
    * and create list of tree elements.<br>
    * Also calculates new preferred size of the tree view
   */
   private void refreshList () {
     // clear old info data
     elementBounds.removeAllElements();
     imagesBounds.removeAllElements();
     treeList.removeAllElements();
     myPrefSize = new Dimension(0, getInitialIndent());
     // go through the tree
     walkWholeTree();
     myPrefSize.height += getInitialIndent();
  }

   /**
    * Refreshes selection - it means removes nodes which
    * cannot be found in new list of items.
    * (usually called after call to refreshList).
   */
   private void refreshSelection () {
     Vector newSelItems = new Vector(10, 10);
     for (int i=0; i<selectedItems.size(); i++) {
       if (treeList.indexOf(selectedItems.elementAt(i))!=-1)
         newSelItems.addElement(selectedItems.elementAt(i));
     }
     selectedItems.removeAllElements();
     selectedItems = newSelItems;
   }

   /**
    * Returns initial y indent
    * (half of preferred size of first visible item)
   */
   private int getInitialIndent () {
		 XFTreeNode firstVisible = tree.getFirstVisible();
     return firstVisible==null ? 0
		        : ((XelfiTreeViewElement)firstVisible.getData()).
              preferredSize().height/2+H_INDENT;
   }

	 private void walkWholeTree () {
		 XFTreeNode root = tree.getRoot();
		 for (int i=0; i<root.getChildCount(); i++) {
				walkTree(root.getChild(i), 
								 0, myPrefSize.height);
		 }
	 }

   /**
    * Walks through one node of tree, calculates its bounds,
    * adds it to the preferred size calculation and to the
    * bounds arrays for faster redrawing.
   */
   private void walkTree (XFTreeNode node, int x, int y) {
		 if (node==null) return;
     treeList.addElement(node);
     XelfiTreeViewElement element = (XelfiTreeViewElement)node.getData();
     Dimension size = element.preferredSize();
     Dimension imageSize = getNodeImageSize(element);
     // calculate preferred size
     myPrefSize.height += Math.max(size.height, imageSize.height) + Y_INDENT;
     if (myPrefSize.width<x+size.width+X_INDENT)
       myPrefSize.width = x+size.width+X_INDENT;
     // add item bounds info array
     elementBounds.addElement(
           new Rectangle(x+2*X_IMAGE_INDENT+imageSize.width,
                         y-size.height/2-1, size.width+1, size.height+2));
     // add image bounds info
     imagesBounds.addElement(
           new Rectangle(x+X_IMAGE_INDENT, y-imageSize.height/2-1,
                         imageSize.width, imageSize.height+2));
     if (element.isPulled()!=true) return;
     // walk through the children
     for (int i=0; i<node.getChildCount(); i++) {
       walkTree(node.getChild(i),
                 x+X_INDENT+TREE_INDENT+1, myPrefSize.height);
     }
   }

   private void paintTree (Graphics g, XFTreeNode node, int x, int y) {
     XelfiTreeViewElement element = (XelfiTreeViewElement)node.getData();
     // get node image size
     Dimension imageSize = getNodeImageSize(element);
     Dimension size = element.preferredSize();
     // if no intersection with visible rectangle, skip drawing
     if (visibleRect.intersects(
           new Rectangle(x+X_IMAGE_INDENT,
                         y-Math.max(imageSize.height, size.height)/2-1,
                         X_IMAGE_INDENT+imageSize.width+size.width,
                         Math.max(imageSize.height, size.height)+2))) {
       // paint node image
       paintNodeImage(g, element, x+X_IMAGE_INDENT, y);
       // paint the element - selected or not selected
       int itemIndex = treeList.indexOf(node);
       Rectangle itemRect = (Rectangle)elementBounds.elementAt(itemIndex);
       if (isSelected(itemIndex))
         element.paintSelected(g, moveX+itemRect.x,
                               moveY+itemRect.y+itemRect.height/2);
       else
         element.paint(g, moveX+itemRect.x,
                         moveY+itemRect.y+itemRect.height/2);
     }
/*     // add size info into bounds array
     elementBounds.addElement(new Rectangle(x+2*X_IMAGE_INDENT+imageSize.width,
                                            y-size.height/2-1,
                                            size.width+1, size.height+2));
     imagesBounds.addElement(
           new Rectangle(x+X_IMAGE_INDENT, y-imageSize.height/2-1,
                         imageSize.width, imageSize.height+2)); */
     int yMove = Math.max(size.height, imageSize.height);
     globalY += yMove + Y_INDENT;
     if (element.isPulled()!=true) return;
     int curx = x+X_IMAGE_INDENT+imageSize.width/2-1;
     int cury = y+imageSize.height/2;
     for (int i=0; i<node.getChildCount(); i++) {
       // paint tree lines
       treeLines(g, curx, cury, curx+TREE_INDENT, globalY);
       cury = globalY;
       // paint children (sub-trees)
       paintTree(g, node.getChild(i), curx+TREE_INDENT+1, globalY);
     }
   }

   /**
    * Paints the shape "L" between points x1, y1 and x2, y2
   */
   private void treeLines (Graphics g, int x1, int y1, int x2, int y2) {
     Color oldColor = g.getColor();
     g.setColor(Color.white);
     g.drawLine(x1+1, y1, x1+1, y2-1);
     g.drawLine(x1, y2-1, x2, y2-1);
     g.setColor(Color.darkGray);
     g.drawLine(x1, y1, x1, y2);
     g.drawLine(x1, y2, x2, y2);
     g.setColor(oldColor);
   }

   /**
    * Helper function for getting the image from file.
   */
  private Image getImage(String imageFile) {
    return(Toolkit.getDefaultToolkit().getImage(imageFile));
  }

   /**
    * Waits until all images needed for proper drawing are loaded.
   */
   private void waitForImages () {
     if (!imagesLoaded) {
       try {
         tracker.waitForAll();
       }
       catch (InterruptedException i) {};
     }
     if (!tracker.isErrorAny()) {
       imagesLoaded = true;
     } else {
       System.err.println("Error: Can't load tree view node images....");
     }
   }

   /**
    * Adds specified image to the Media Tracker and
    * waits until it's loaded.
   */
   private boolean waitForImage (Image image) {
     tracker.addImage(image, imageIndex);
    try {
       tracker.waitForID(imageIndex);
     }
    catch (InterruptedException i) {};
     imageIndex++;
     return !tracker.isErrorID(imageIndex-1);
   }

   /**
    * Create images and add them to the tracker.
    * Note: Tracker has to be created already.
    * Note2: Must be called before first call to refreshList()!!!
   */
   private void addImages () {
     folderImage = getImage(getImagePath("folder.gif"));
     openFolderImage = getImage(getImagePath("openfolder.gif"));
     nodeImage = getImage(getImagePath("node.gif"));
    tracker.addImage(folderImage, 1);
    tracker.addImage(openFolderImage, 2);
    tracker.addImage(nodeImage, 3);
     imageIndex = 4;
   }


   /**
    * Returns image which belongs to the specified element
    * in its current state.
   */
   private Image getElementImage (XelfiTreeViewElement element) {
     if (element.canHaveChildren()) {
       if (element.isPulled())
         return openFolderImage;
       else
         return folderImage;
     } else {
         return nodeImage;
     }
   }

   /**
    * Returns size of image which belongs to the element
    * in its current state.
   */
   private Dimension getNodeImageSize (XelfiTreeViewElement element) {
     Image elementImage = getElementImage(element);
     return new Dimension(elementImage.getWidth(parentComp),
                          elementImage.getHeight(parentComp));
   }

   /**
    * Paints image which belongs to the specified node in
    * its current state.
    * x,y are coordinates of x-left y-middle point.
   */
   private void paintNodeImage(Graphics g, XelfiTreeViewElement element,
                               int x, int y) {
     Image elementImage = getElementImage(element);
     int imageHeight = elementImage.getHeight(parentComp);
     g.drawImage(elementImage, x, y-imageHeight/2, parentComp);
   }

   /**
    * Creates and returns full path of the image file from
    * the filename.
   */
   private String getImagePath(String fileName) {
     String sep = new String(System.getProperty("file.separator"));
     StringBuffer buf = new StringBuffer(XelfiPath.get() + sep +
       "awt" + sep + fileName);
     return new String(buf);
   }

}


