
package xelfi.browser.util;
import java.awt.*;
/**
 * Flow layout is used to layout buttons in a panel. It will arrange
 * buttons left to right until no more buttons fit on the same line.
 * Each line is centered.
 *
 * @version    1.18, 14 Dec 1995
 * @author     Arthur van Hoff
 * @author     Sami Shaio
 */
public class DLayout implements LayoutManager {

    int hgap;
    int vgap;

    /**
     * Constructs a new Flow Layout with a centered alignment.
     */
    public DLayout() {
   this(5, 5);
    }

    /**
     * Constructs a new Flow Layout with the specified alignment and gap
     * values.
     * @param hgap the horizontal gap variable
     * @param vgap the vertical gap variable
     */
    public DLayout(int hgap, int vgap) {
   this.hgap = hgap;
   this.vgap = vgap;
    }

    /**
     * Adds the specified component to the layout. Not used by this class.
     * @param name the name of the component
     * @param comp the the component to be added
     */
    public void addLayoutComponent(String name, Component comp) {
    }

    /**
     * Removes the specified component from the layout. Not used by
     * this class.
     * @param comp the component to remove
     */
    public void removeLayoutComponent(Component comp) {
    }

    /**
     * Returns the preferred dimensions for this layout given the components
     * in the specified target container.
     * @param target the component which needs to be laid out
     * @see Container
     * @see #minimumLayoutSize
     */
    public Dimension preferredLayoutSize(Container target) {
   Dimension dim = new Dimension(0, 0);
   int nmembers = Math.min(target.countComponents(),2);

   for (int i = 0 ; i < nmembers ; i++)
   {
       Component m = target.getComponent(i);
       if (m.isVisible())
       {
           Dimension d = m.preferredSize();
           dim.height = Math.max(dim.height, d.height);
           if (i > 0)
           {
               dim.width += vgap;
           }
           dim.width += d.width;
       }
   }
   Insets insets = target.insets();
   dim.width += insets.left + insets.right + hgap*2;
   dim.height += insets.top + insets.bottom + vgap*2;
   return dim;
    }

    /**
     * Returns the minimum dimensions needed to layout the components
     * contained in the specified target container.
     * @param target the component which needs to be laid out
     * @see #preferredLayoutSize
     */
    public Dimension minimumLayoutSize(Container target)
    {
       Dimension dim = new Dimension(0, 0);
       int nmembers = Math.min(target.countComponents(),2);

       for (int i = 0 ; i < nmembers ; i++)
       {
           Component m = target.getComponent(i);
           if (m.isVisible())
           {
               Dimension d = m.minimumSize();
               dim.height = Math.max(dim.height, d.height);
               if (i > 0)
               {
                   dim.width += vgap;
               }
               dim.width += d.width;
           }
       }
       Insets insets = target.insets();
       dim.width += insets.left + insets.right + hgap*2;
       dim.height += insets.top + insets.bottom + vgap*2;
       return dim;
    }

    /**
     * Lays out the container. This method will actually reshape the
     * components in the target in order to satisfy the constraints of
     * the BorderLayout object.
     * @param target the specified component being laid out.
     * @see Container
     */
    public void layoutContainer(Container target)
    {
        Insets insets = target.insets();
        int maxheight = target.size().height - (insets.top +
            insets.bottom + vgap*2);
        int nmembers = target.countComponents();
        int x = insets.left + hgap, y = 0;
        int roww = 0, start = 0;

        Component m = target.getComponent(0);
        if (m.isVisible())
        {
            Dimension d = m.preferredSize();
            m.reshape(insets.left + hgap,insets.top + vgap,
                 d.width, d.height);
        }
        m = target.getComponent(1);
        if (m.isVisible())
        {
            Dimension d = m.preferredSize();
             m.reshape(target.size().width - (insets.left + hgap + d.width),
                 insets.top + vgap, d.width, d.height);
        }
    }

    /**
     * Returns the String representation of this FlowLayout's values.
     */
    public String toString()
    {
       String str = "";
       return getClass().getName() + "[hgap=" + hgap + ",vgap=" +
           vgap + str + "]";
    }
}

