/** XelfiCommpiler class
* @author: Filip Dvorak
* @version: 1.3
*/

package xelfi.compiler;

import java.util.*;
import java.io.*;

import xelfi.editor.*;
import xelfi.xedit.*;
import xelfi.top.TopLevel;
import xelfi.util.*;

  /** This class contains methods for compilation that can be called from other packages. */
public class XelfiCompiler implements Runnable, FilenameFilter
{
  private static final int COMPILE_PACKAGE = 1;
  private static final int ABORT_PACKAGE = 2;
  private static final int SCAN_PACKAGE = 3;

    /** Compile classes. New thread is created for the compilation. 
      @param classes Vector whose elements are Strings, contain names of classes to compile
    */
  public static void compileClasses(Vector classes)
  {
    newSession(classes).start();
  }

    /** Compile one class. New thread is created for the compilation. 
      @param className name of a class to compile
    */
  public static void compileClass(String className)
  {
    Vector cl = new Vector(1);
    cl.addElement(className);
    newSession(cl).start();
  }


    /** Compile all action. The action depends on the current compiler settings in the CompilerOptions.
    */
  public static void compileAll()
  {
    newSession(null).start();
  }

    /** Creates a new session - new Thread, and does all things neede to de before compilation. */
  protected static Thread newSession(Vector classToCompile)
  {
    try
    {
        Editor.saveAll();
    }
    catch(IOException e) { new MessageBox("Error", "Exception occured when saving.").show(); }
    TopLevel.getDebugger().finishDebugging();
    TopLevel.resetSnapshot();
    Thread sessionThread = new Thread(new XelfiCompiler(classToCompile));
    sessionThread.setPriority(Thread.MIN_PRIORITY);
    return sessionThread;
  }

  private Vector toCompile;
  private Output outWin;
  private MainCompiler main;
  private XelfiCompilerOutput outputStream;

    // for data accesing from two methods
  private Vector subDirs;

    /** Called when creation new session. */
  protected XelfiCompiler(Vector classesToCompile)
  {
    outWin = TopLevel.getOutput();
    outWin.clear();
    TopLevel.bringToTop(outWin, true);
    outputStream = new XelfiCompilerOutput();
    if(CompilerOptions.getUseInternalCompiler()) main = new XelfiCompilerMain(outputStream);
    else main = new ExternalCompiler(outputStream);
    toCompile = classesToCompile;
  }

    /** Runable implementation. */
  public void run()
  {
    if(toCompile != null)  // compile only specified classes
    try
    {
      for(int i = 0; i < toCompile.size(); i++)
      {
        XelfiFile cls = XelfiFile.find((String)toCompile.elementAt(i), false);
        main.compileClass(cls.getFile());
      }
    }
    catch(IOException e) { new MessageBox("Error", e.toString()).show(); }
    else if(CompilerOptions.onCompileAll() == CompilerOptions.ALL_EXCEPT)
    {
      Enumeration paths = PathScanner.elements(System.getProperty("java.class.path"));
      while(paths.hasMoreElements())
      {
        File root = new File((String)paths.nextElement());
        if(root.exists() && root.isDirectory()) compilePackage("", root);
      }
    }
    else if(CompilerOptions.onCompileAll() == CompilerOptions.LISTED_ONLY)// compile specified packeges
    {
      Enumeration packList = CompilerOptions.getPackageList();
      while(packList.hasMoreElements())
      {
        String pack = (String)packList.nextElement();
        File packDir = getPackageFile(pack);
        if(packDir != null) compilePackage(pack, packDir);
        else ; // report an error
      }
    }
    else compileEditors();// editor windows only

    outWin.print("Compilation finished");
    outWin.flush();
  }

    /** Compiles all opened editor windows. */
  protected void compileEditors()
  {
    Enumeration eds = TopLevel.allEditors();
    while(eds.hasMoreElements())
    {
      JavaEditorFrame ed = (JavaEditorFrame)eds.nextElement();
      File javaFile = ed.getSource().getFile().getFile();
      File classFile = getClassFile(javaFile);
      if((!classFile.exists() || classFile.lastModified() < javaFile.lastModified()) &&
        !outputStream.errorHistoryContains(ed.getSource().getName())) main.compileClass(javaFile);
    }
  }

  protected File getClassFile(File javaF)
  {
    String javaName = javaF.toString();
    return new File(javaName.substring(0, javaName.length() - 5) + ".class");  // classname
  }


  protected File getPackageFile(String packName)
  {
// System.out.println(packName + " -> ");

      // a.b.c -> a/b/c
    StringBuffer relativePathBuf = new StringBuffer(40);
    StringTokenizer t = new StringTokenizer(packName, ".");
    while(t.hasMoreTokens())
    {
      relativePathBuf.append(t.nextToken());
      if(t.hasMoreTokens()) relativePathBuf.append(File.separator);
    }
    String relativePath = relativePathBuf.toString();
// System.out.println(relativePath);
      // try to appand it to the classpath elements
    Enumeration paths = PathScanner.elements(System.getProperty("java.class.path"));
    while(paths.hasMoreElements())
    {
      String path = PathScanner.addFileSeparator((String)paths.nextElement());
// System.out.println("trying " + path + relativePath );

      File packDir = new File(path + relativePath);
      if(packDir.exists() && packDir.isDirectory()) 
       return packDir;  

    }
//    System.out.println("NULL");
    return null;
  }

  protected void compilePackage(String name, File dir)
  {
//    System.out.println("compile package " + name + "   " + dir);
    int action;
    if((action = whatAction(name)) == ABORT_PACKAGE) return;
    String dirPath = PathScanner.addFileSeparator(dir.toString());
    String packExtend = name.equals("") ? "" : (name + ".");

    subDirs = new Vector(20);
    String javaFiles[] = dir.list(this);  // javaFiles will contain .java files
    // make this package
    if(action == COMPILE_PACKAGE)
    {
      int len = javaFiles.length, i;
      for(i = 0; i < len; i++)
      {
        String jfile = javaFiles[i];  // classname.java
        String clname = jfile.substring(0, jfile.length() - 5);  // classname
        String clsfilename = clname + ".class";  // classname.class
        File clsfile = new File(dir, clsfilename);
        File jfileFile = new File(dir, jfile);
        if((!clsfile.exists() || clsfile.lastModified() < jfileFile.lastModified()) &&
          !outputStream.errorHistoryContains(packExtend + clname))
            main.compileClass(jfileFile);
      }
    }
    javaFiles = null;  // Garbage collecting enabled before recursion
    Vector thisSubdirs = subDirs;  // this vectors belongs to local variable
    subDirs = null;
      // compile subpackeges
    String prefixPackage = name.equals("") ? "" : (name + ".");
    for(int i = 0; i < thisSubdirs.size(); i++)
      compilePackage(prefixPackage + ((File)thisSubdirs.elementAt(i)).getName(), (File)thisSubdirs.elementAt(i));
  }

  public boolean accept(File dir, String name)
  {
    int dot = name.indexOf('.');
      // .java file ?
    if(dot != -1 && name.substring(dot).equals(".java")) return true;
      // subdirectory ?
    File f = new File(dir, name);
    if(f.isDirectory()) subDirs.addElement(f);
      // .class file ?
    return false;
  }

/*  private int whatAction(String packName)
  {
     int q = wwhatAction(packName);
     if(q == SCAN_PACKAGE) System.out.println("SCAN_PACKAGE");
     if(q == ABORT_PACKAGE) System.out.println("ABORT_PACKAGE");
     if(q == COMPILE_PACKAGE) System.out.println("COMPILE_PACKAGE");
     return q;
  }
*/

  private int whatAction(String packName)
  {
//System.out.println("What action " + packName + "  recursive: " + CompilerOptions.packListRecursive());

    if(CompilerOptions.onCompileAll() == CompilerOptions.ALL_EXCEPT)
      if(CompilerOptions.packListContains(packName))
        if(CompilerOptions.packListRecursive()) return ABORT_PACKAGE;
        else return SCAN_PACKAGE;
      else return COMPILE_PACKAGE;
    else // compile listed pacakges
      if(CompilerOptions.packListContains(packName)) return COMPILE_PACKAGE;
      else return CompilerOptions.packListRecursive() ? COMPILE_PACKAGE : ABORT_PACKAGE;
  }
}


