/**
 * BreakpointsView
 *
 * @author Roman Blazevic
 * @version 970225
 */

package xelfi.debugger;

import java.awt.*;
import xelfi.top.TopLevel;
import xelfi.top.TopDialog;

/**
 * This is a view on data stored in the document class Breakpoints.
 */

public class BreakpointsView extends TopDialog
{
	private Breakpoints document;
	private List list;
	private BreakpointDialog dlg;

	static final int HINT_COMPLETE_UPDATE = 0;
	static final int HINT_INSERTED = 1;
	static final int HINT_REMOVED = 2;
	static final int HINT_CHANGED = 3;

	static final int KEY_ENTER = 10;
	static final int KEY_DELETE = 127;

	private static final int PROCESSING_NO_DIALOG = 0;
	private static final int PROCESSING_INSERT_DIALOG = 1;

	private int state;
	private int lineStamp;

	/**
	 * BreakpointsView constructor.
	 *
	 * breakpoints - the document for this view
	 */
	BreakpointsView(Breakpoints breakpoints)
	{
		super(TopLevel.getMenu(), false);
		setTitle("Breakpoints");
		//super("Breakpoints");
		setBackground(Color.lightGray);

		state = PROCESSING_NO_DIALOG;

		document = breakpoints;
		list = new List();
		add("Center", list);
		dlg = null;
	}

	/**
	 * My handleEvent() implementation.
	 */
	public boolean handleEvent(Event event)
	{
		int selectedIndex = list.getSelectedIndex();

		if ((event.id == Event.KEY_PRESS && event.key == KEY_ENTER)
			|| (event.id == Event.ACTION_EVENT))
		{
			if (selectedIndex != -1 && selectedIndex != list.countItems()-1)
			{
				document.breakpointChosen(selectedIndex);
				return true;
			}
			else
				if (selectedIndex == list.countItems()-1)
				{
					event.id = Event.KEY_PRESS;
					event.key = 9; // Ctrl+I - insert breakpoint
				}
		}

		if (event.id == Event.KEY_PRESS)
                {
                    if (dlg == null) // dialog is already shown
                        switch (event.key)
                        {
				case 4: // Ctrl+D - disable breakpoint
      					if (selectedIndex != -1 && selectedIndex != list.countItems()-1)
        					document.disableBreakpoint(selectedIndex);
       					break;
                               	case 5: // Ctrl+E - enable breakpoint
	       				if (selectedIndex != -1 && selectedIndex != list.countItems()-1)
       						document.enableBreakpoint(selectedIndex);
       					break;
       				case 9: // Ctrl+I - insert breakpoint
       					dlg = new BreakpointDialog(this, "Insert a new breakpoint");
       					state = PROCESSING_INSERT_DIALOG;
       					lineStamp = list.getSelectedIndex();
       					dlg.show();
       					break;
       				case 18: // Ctrl+R - refresh
       					update(HINT_COMPLETE_UPDATE, -1);
       					break;
       				case KEY_DELETE:
       					if (selectedIndex != -1 && selectedIndex != list.countItems()-1)
       						document.removeBreakpoint(selectedIndex);
                                	break;
                        	default:
                            		return super.handleEvent(event);
			}
			return true;
		}
		return super.handleEvent(event);
	}

	/**
	 * My action() implementation.
	 * This method processes the result sent from the instance of the BreakpointDialogResult.
	 */
	public boolean action(Event event, Object what)
	{
		if (event == null)
		{
			BreakpointDialogResult result;

			result = (BreakpointDialogResult)what;

			if (state == PROCESSING_INSERT_DIALOG)
				if (result.returnCode == BreakpointDialog.ID_OK)
				{
					Breakpoint newBreakpoint = new Breakpoint(result.className, result.lineNumber);

					newBreakpoint.convertLineNumber();
					if (lineStamp != -1)
						document.insertBreakpoint(lineStamp, newBreakpoint);
					else
						document.insertBreakpoint(list.countItems()-1, newBreakpoint);
				}

			state = PROCESSING_NO_DIALOG;
			dlg = null;
			return true;
		}
		return super.action(event, what);
	}

	/**
	 * This method is called when the content of the view must be
	 * updated.
	 */
	void update(int hint, int index)
	{
		if (!isVisible())
			return;

		int numberOfBreakpoints = document.getNumberOfBreakpoints();
		String line;

		switch (hint)
		{
			case HINT_COMPLETE_UPDATE:
				list.clear();
				for(int i = 0; i < numberOfBreakpoints; i++)
				{
					line = " ["+(i+1)+"] "+document.getBreakpoint(i).toString();
					list.addItem(line);
				}
					list.addItem(""); // last line always contains string ""
					//list.select(0);
				break;
			case HINT_INSERTED:
				list.delItems(index, list.countItems()-1);
				for(int i = index; i < numberOfBreakpoints; i++)
				{
					line = " ["+(i+1)+"] "+document.getBreakpoint(i).toString();
					list.addItem(line);
				}
				list.addItem(""); // last line always contains string ""
				list.select(index+1);
				break;
			case HINT_REMOVED:
				list.delItems(index, list.countItems()-1);
				for(int i = index; i < numberOfBreakpoints; i++)
				{
					line = " ["+(i+1)+"] "+document.getBreakpoint(i).toString();
					list.addItem(line);
				}
				list.addItem(""); // last line always contains string ""
				list.select(index);
				break;
			case HINT_CHANGED:
				line = " ["+(index+1)+"] "+document.getBreakpoint(index).toString();
				list.replaceItem(line, index);
				list.select(index);
				break;
			default:
				System.out.println("unsupported hint");
				break;
		}
	}

	/**
	 * My show() implementation.
	 */
	public void show()
	{
		super.show();
		list.requestFocus();
		document.breakpointsViewShown();
	}
}
