/**
* Xelfi
*/

package xelfi.design.forms;

import java.util.Vector;
import java.io.*;
import xelfi.editor.*;
import xelfi.design.components.*;
import xelfi.design.util.XFDesignOptions;
import xelfi.util.Debug;
import xelfi.util.MessageBox;
import xelfi.util.MessageBoxListener;

/**
* Class that manages events for one form.
*
* @version 	1.14, 06 Apr 1997
* @author 		Ian Formanek
*/


class EventPool implements MessageBoxListener {
	private Position eventsPos;
	private Position handlerPos;
	private Block headerBlock, endBlock;
	private Vector events;
	
	private final static int ASK_NONE = 0;
	private final static int ASK_DELETE = 1;
	private int state = ASK_NONE; // message box listener state;
	private EventInfo to_delete=null; // event which is to be deleted (asking user for)
	
	EventPool(Position aEventsPos, Position aHandlerPos) {
		events = new Vector(1,1);
		try {
			endBlock = Block.text(aEventsPos, "// End of event handlers\n\n", new GuardedStamp());
			headerBlock = Block.text(endBlock.getBegin(), "// Event handlers\n\n", new GuardedStamp());
		} catch (NotAllowedException e) {
			throw new InternalError("Block generation error");
		}
		eventsPos = endBlock.getBegin();
		handlerPos = aHandlerPos;
	}

	public Position getBegin() { return headerBlock.getBegin(); }
	public Position getEnd() { return endBlock.getEnd(); }
	
	private EventInfo findEvent(String evName) {
		int evCount = events.size();
		for (int i=0; i<evCount; i++) {
			EventInfo ei = (EventInfo)events.elementAt(i);
			if (evName.equalsIgnoreCase(ei.evName)) 
				return ei;
		}
		return null;
	}
	
	public Block assignEvent(XelfiDesignComponent xdc, int index, String evName) {
		EventInfo ei = findEvent(evName);
		Block userBlock = null;
		if (ei == null) {
			EventBlock evBlock = EventBlock.create(eventsPos, evName);
			eventsPos = evBlock.getBegin();
			ei = new EventInfo(evName, evBlock);
			events.addElement(ei);
			userBlock = evBlock.ev_body;
		}

		// we create new block into handleEvent for this component
		Block hBlock = null;
		try {
			hBlock = Block.text(handlerPos, 
				             XFDesignOptions.indent(2)+"if ("+xdc.generateEventCondition(index)+")\n"+
				             XFDesignOptions.indent(3)+"return "+ei.evName+"("+XFDesignOptions.eventVarName+");\n",
									new GuardedStamp());
			handlerPos = hBlock.getBegin();
		} catch (NotAllowedException e) {
			throw new InternalError("Block generation error");
		}

		ei.acquire(xdc, index, hBlock);
		
		return userBlock;
	}

	public void renameEvent(String oldName, String newName) {
		if (oldName.equals(newName)) return;
		EventInfo ei = findEvent(oldName);
		if (ei != null) {
			EventInfo ei2 = findEvent(newName);
			if (ei2 != null) // there already exists event handler with the new name
				new MessageBox("Event attaching error", 
							"The event handler <"+newName+"> already exists.\n"+
							"You must first remove the old event handler before you can attach to the new one"
							).show();
			else	ei.rename(newName);
		}
	}

	public void componentRenamed(XelfiDesignComponent xdc) {
		int evCount = events.size();
		for (int i=0; i<evCount; i++)
			((EventInfo)events.elementAt(i)).componentRenamed(xdc);
	}

	public void releaseEvent(XelfiDesignComponent xdc, int index) {
		EventInfo ei = findEvent(xdc.getEvent(index));
		if (ei != null) {
			Debug.debug(Debug.FORM, "releasing ei");
			if (ei.release(xdc, index)) {
				Debug.debug(Debug.FORM, "last ei released");
				new MessageBox("Delete event handler?", 
							"The event handler <"+ei.evName+"> is not attached to any component anymore.\n"+
							"Do you want to delete it?\n"+
							"WARNING: All the code you might have writen inside it will be lost.",
							MessageBox.YESNO, this).show();
				state = ASK_DELETE;
				to_delete = ei;
			}
		}
	}

	public String[] getDefinedEvents() {
		int evCount = events.size();
		String list[] = new String[evCount];
		for (int i=0; i<evCount; i++)
			list[i] = new String(((EventInfo)events.elementAt(i)).evName);			
		return list;
	}

	public Block getEventUserBlock(String evName) {
		EventInfo ei = findEvent(evName);
		if (ei == null) return null;
		return ei.evBlock.ev_body;
	}
	
	public void loadFromStream(DataInput stream) throws
	IOException {
		int count = stream.readInt();
		Debug.debug(Debug.FORMIO_LOAD, "loading evPool: eventInfos: "+count);
		for (int i=0; i<count; i++) {
			EventBlock evBlock = EventBlock.create(eventsPos, "");
			eventsPos = evBlock.getBegin();
			EventInfo ei = new EventInfo("", evBlock);
			ei.loadFromStream(stream);
			events.addElement(ei);
		}
	}

	public void saveToStream(DataOutput stream) throws
	IOException {
		int count = events.size();
		Debug.debug(Debug.FORMIO_SAVE, "saving evPool: eventInfos: "+count);
		stream.writeInt(count);
		for (int i=count-1; i>=0; i--)
			((EventInfo)events.elementAt(i)).saveToStream(stream);
	}
	
	public void messageNotify(int message) {
		if (state == ASK_DELETE && to_delete != null && message==MessageBox.YES) {
			to_delete.evBlock.delete();
			events.removeElement(to_delete);
		}
		state = ASK_NONE;
		to_delete = null;
	}
}

class EventInfo {
	String evName;
	EventBlock evBlock;
	Vector compInfos;

	EventInfo(String aEvName, EventBlock aEvBlock) {
		evName = aEvName;
		evBlock = aEvBlock;
		compInfos = new Vector(1,1);
	}
	
	void rename(String newName) {
		evName = newName;
		evBlock.rename(evName);
		int count = compInfos.size();
		for (int i=0; i<count; i++) {
			CompInfo ci = (CompInfo)compInfos.elementAt(i);
			ci.xdc.setEvent(ci.index, newName);
			try {
				ci.handleBlock.set(XFDesignOptions.indent(2)+"if ("+ci.xdc.generateEventCondition(ci.index)+")\n"+
			                     XFDesignOptions.indent(3)+"return "+evName+"("+XFDesignOptions.eventVarName+");\n",
						new GuardedStamp());
			} catch (NotAllowedException e) {
				throw new InternalError("Block generation error");
			}
		}
	}
	
	void componentRenamed(XelfiDesignComponent xdc) {
		int count = compInfos.size();
		for (int i=0; i<count; i++) {
			CompInfo ci = (CompInfo)compInfos.elementAt(i);
			if (ci.xdc == xdc) {
				try {
					ci.handleBlock.set(XFDesignOptions.indent(2)+"if ("+ci.xdc.generateEventCondition(ci.index)+")\n"+
				                        XFDesignOptions.indent(3)+"return "+evName+"("+XFDesignOptions.eventVarName+");\n",
							new GuardedStamp());
				} catch (NotAllowedException e) {
					throw new InternalError("Block generation error");
				}
			}
		}
	}

	void acquire(XelfiDesignComponent xdc, int index, Block hBlock) {
		compInfos.addElement(new CompInfo(xdc, index, hBlock));
		xdc.setEvent(index, evName);
	}		

	boolean release(XelfiDesignComponent xdc, int index) {
		int count = compInfos.size();
		for (int i=0; i<count; i++) {
			CompInfo ci = (CompInfo)compInfos.elementAt(i);
			if (ci.xdc == xdc && ci.index == index) {
				ci.xdc.setEvent(ci.index, null);
				try {
					ci.handleBlock.delete(new GuardedStamp());
				} catch (NotAllowedException e) {
					throw new InternalError("Block generation error");
				}
				compInfos.removeElementAt(i);
				break;
			}
		}
		return  (compInfos.size() == 0);
	}		

	public void loadFromStream(DataInput stream) throws
	IOException {
		evName = stream.readUTF();
		evBlock.rename(evName);
		evBlock.loadFromStream(stream);
		// the references (compInfo) are restored by components themselves
	}

	public void saveToStream(DataOutput stream) throws
	IOException {
		stream.writeUTF(evName);
		evBlock.saveToStream(stream);
		// the references (compInfo) are stored by components themselves
	}
}	

class CompInfo {
	XelfiDesignComponent xdc;
	int index;
	Block handleBlock;
	CompInfo(XelfiDesignComponent aXdc, int aIndex, Block aHandleBlock) {
		xdc = aXdc;
		index = aIndex;
		handleBlock = aHandleBlock;
	}
}
