package xelfi.design.inspector;

import java.awt.*;
import java.util.Vector;

import xelfi.design.components.*;
import xelfi.design.util.*;
import xelfi.top.TopLevel;
import xelfi.util.Debug;

/**
 * Class XFPropertyPanel. It's main class for property
 * manipulation in component inspector.
 *
 * @author Dafe
 * @version 1.26, 2 Dec 1996
   Last change:  DF   16 Apr 97   11:08 am
*/
class XFPropertyPanel extends Panel {

   // drawing constants
   private static final int    BORDER = 4;
   private static final int    SEPARATOR_SIZE = 4;

   private XFNamesPanel        names;
   private XFValuesPanel         values;
  private Scrollbar           scrollbar;
   private XFCommonProperties  commonProps;
   private boolean               laidOut;
   private Dimension             oldSize = null;

   private XFPropertyStringBox   selectedStringBox;
   private XFPropertyValueBox  selectedValueBox;

   /**
    * Constructor, takes common set of properties
    * and creates proper content.
   */
   XFPropertyPanel (XFCommonProperties commonProps) {
     // layouts sub-panels in absolute layout
     setLayout(null);
     laidOut = false;
     // creates sub-panels
     names = new XFNamesPanel();
     values = new XFValuesPanel(commonProps);
    scrollbar = new Scrollbar();
    add(scrollbar);
     add(names);
     add(values);
     this.commonProps = commonProps;
     if (commonProps!=null) fillIn(commonProps);
   }

   /**
    * Sets new set of properties common to selected components
    * into property panel.
    * Actual page remains unchanged, no property is selected.
    * Property panel properly repaints itself.
   */
   synchronized void changeContent (XFCommonProperties commonProps) {
     this.commonProps = commonProps;
     // clears current content
     empty();
     if (commonProps==null) return;
     // refills both panels with property names and values
     fillIn(commonProps);
     repaint();
   }

   /**
    * Empties names and values of properties property panel.
   */
   void empty () {
     names.removeAllProps();
     values.removeAllProps();
   }

   /**
    * Refreshes values in property value panel
   */
   synchronized void refreshValues () {
     values.refresh();
   }

   /**
    * Responds to user actions.
   */
  public boolean handleEvent (Event e) {
    boolean result = super.handleEvent(e);
     switch (e.id) {
       case XFNamesPanel.SELECTION_CHANGED:
         boxClicked((XFPropertyStringBox)e.arg,
                     e.arg==null ? null
                       : (XFPropertyValueBox)(values.getProperty(
                             ((XFPropertyStringBox)e.arg).getIndex())));
         break;
       case XFNamesPanel.SELECTION_DOUBLE_CLICKED:
         values.setDefaultEventName((XFPropertyValueBox)(values.getProperty(
                                    ((XFPropertyStringBox)e.arg).getIndex())));
         break;
       case XFValuesPanel.PROPERTY_CHANGED:
         boxClicked(e.arg==null ? null
                       : (XFPropertyStringBox)(names.getProperty(
                             ((XFPropertyValueBox)e.arg).getIndex())),
                     (XFPropertyValueBox)e.arg);
         break;
      case XFValuesPanel.SPECIAL_DIALOG:
        runSpecialDialog(((XFPropertyStringBox)e.arg).getIndex());
       case XFValuesPanel.DONE:
         boxReleased();
/*        e.arg==null ? null
                       : (XFPropertyStringBox)(names.getProperty(
                             ((XFPropertyValueBox)e.arg).getIndex())),
                     (XFPropertyValueBox)e.arg); */
         break;
       case XFValuesPanel.ESCAPE:
         boxEscaped();
         break;
      case Event.SCROLL_LINE_UP:
      case Event.SCROLL_LINE_DOWN:
      case Event.SCROLL_PAGE_UP:
      case Event.SCROLL_PAGE_DOWN:
      case Event.SCROLL_ABSOLUTE:
//        Debug.debug(Debug.INSPECTOR, "scroll event...");
        // scrollbar action event catched
        // set upper view point for names and values
        int value = scrollbar.getValue();
        names.setStartingPoint(0, value);
        values.setStartingPoint(0, value);
        names.repaint();
        values.repaint();
        break;
     }
     return result;
   }

  /**
   * Responds to incoming key events.
  */
  public boolean keyDown (Event e, int key) {
    switch (key) {
      case Event.UP:
        selectPreviousBox();
        return true;
      case Event.DOWN:
        selectNextBox();
        return true;
    }
    return false;
  }

   /**
    * Paints client area of panel
   */
   public void paint (Graphics g) {
     super.paint(g);
     Dimension   mySize = size();

     drawBorder(g, mySize);

    int namesHeight = names.preferredHeight();
    boolean enableScroll = namesHeight>mySize.height;
    setScrollbar(enableScroll, mySize, namesHeight);

    // modify bounds if scrollbar is enabled
    int scrollbarWidth = scrollbar.preferredSize().width;
    if (enableScroll)
      mySize.width -= scrollbarWidth;

     int columnWidth = (mySize.width-1-2*BORDER-SEPARATOR_SIZE)/2;
     int columnHeight = mySize.height-1-2*BORDER;
     drawSeparator(g, columnWidth, mySize);

     laidOut = ((oldSize!=null) && ((mySize.width==oldSize.width) &&
         (mySize.height==oldSize.height)));
     if (!laidOut) {
       names.reshape(BORDER, BORDER, columnWidth, columnHeight);
       values.reshape(BORDER+columnWidth+SEPARATOR_SIZE,
                       BORDER, columnWidth, columnHeight);
      if (enableScroll)  {
        scrollbar.reshape(mySize.width-BORDER, BORDER,
                          scrollbarWidth, columnHeight);
        // find out height of boxes....
        // (subpanels are not empty now, 'cause scrollbar is shown..)
//        int boxHeight = names.getProperty(0).preferredSize().height;
//         Debug.debug(Debug.INSPECTOR, "line increment is: "+scrollbar.getLineIncrement());
/*        if (scrollbar.getLineIncrement()!=boxHeight) {
          scrollbar.setLineIncrement(boxHeight);
        } */
      }
       oldSize = mySize;
     }
     names.repaint();
     values.repaint();
   }

  /**
   * Shows (and sets values) or hides scrollbar.
  */
  private void setScrollbar (boolean enableScroll, Dimension mySize,
                             int namesHeight) {
     int visibleHeight = mySize.height-2*BORDER;
     int boxHeight = names.getBoxHeight();
    if (enableScroll) {
      scrollbar.setValues(names.getStartingPoint().y, 
                           visibleHeight/boxHeight+1,
                          0, (namesHeight-visibleHeight)/boxHeight+1);
      scrollbar.show();
    } else {
      scrollbar.hide();
      Point startingPoint = names.getStartingPoint();
      if ((startingPoint.x!=0)||(startingPoint.y!=0)) {
        // set position to 0, 0
        names.setStartingPoint(new Point(0, 0));
        values.setStartingPoint(new Point(0, 0));
      }
    }
  }

   /**
    * Method called only from handleevent. Performs a response to user
    * click on string box
   */
   private void boxClicked (XFPropertyStringBox stringBox,
                             XFPropertyValueBox valueBox) {
    XFPropertyStringBox curString = names.getSelection();
    XFPropertyValueBox curValue = (XFPropertyValueBox)values.getSelection();

     if (valueBox!=curValue) {
       if (curValue!=null) {
         // don't change old selection if new value wasn't accepted
        if (!values.saveValue()) return;
        curValue.up();
      }
    }

     if (stringBox!=curString) {
       if (curString!=null) curString.up();
    }

    if (stringBox!=curString) {
       // select new string box
      names.setSelection(stringBox);
      stringBox.down();
       names.repaint();
    }
    if (valueBox!=curValue) {
      // select new value box
      values.requestFocus();
      values.setSelection(valueBox);
      valueBox.down();
       values.repaint();
    }
   }

   /**
    * Release selection of the boxes and forces property update
    * (by calling saveValue on value box)
   */
   private void boxReleased () {
     // unselects selected string and value boxes
     if (names.getSelection()!=null) {
       // stay selected if new value wasn't accepted
       if (!values.saveValue()) return;
       names.getSelection().up();
       // values.saveValue();
       values.getSelection().up();
     }
     // set selected boxes to none
    names.clearSelection();
    values.clearSelection();
    names.repaint();
   }

   /**
    * Release selection of the boxes and without property updating
   */
   private void boxEscaped () {
     // unselects selected string and value boxes
     if (names.getSelection()!=null) {
       names.getSelection().up();
       values.getSelection().up();
     }
     // set selected boxes to none
    names.clearSelection();
    values.clearSelection();
    names.repaint();
  }

  /**
   * Selects boxes next to currently selected.
  */
  private void selectNextBox () {
    boxClicked(names.nextToSelected(),
               (XFPropertyValueBox)values.nextToSelected());
    makeSelectedVisible();
  }

  /**
  * Selects boxes previous to currently selected.
  */
  private void selectPreviousBox () {
    boxClicked(names.previousToSelected(),
               (XFPropertyValueBox)values.previousToSelected());
    makeSelectedVisible();
  }

  /**
   * Checks if selected boxes are visible and performs
   * additional scroll up/dowm if it's needed.
  */
  private void makeSelectedVisible () {
    XFPropertyStringBox selName = names.getSelection();
    XFPropertyStringBox selValue = values.getSelection();
    if ((selName==null)||(selValue==null)) return;
    // get selected box's bounds
    Dimension mySize = size();
    Rectangle bounds = names.calculateBounds(selName);
    // decide whether element is scrolled out or not
    if ((bounds.y>=0)&&(bounds.y+bounds.height+2*BORDER<=mySize.height))
      return;
//    Point namesPoint = names.getStartingPoint();
//    Point valuesPoint = values.getStartingPoint();
     values.moveControls();
     if (bounds.y<0) {
       names.bringBoxToTop(selName);
       values.bringBoxToTop(selValue);
     } else {
       names.bringBoxToDown(selName, mySize.height-2*BORDER);
       values.bringBoxToDown(selValue, mySize.height-2*BORDER);
     }
/*    int diff = bounds.y<0 ? bounds.y
                          : (bounds.y+bounds.height+2*BORDER)-mySize.height;
    names.setStartingPoint(namesPoint.x, namesPoint.y+diff);
    values.setStartingPoint(valuesPoint.x, valuesPoint.y+diff); */
    syncScrollbar();
    names.repaint();
    values.repaint();
  }

  /**
  * Synchronize scrollbar's positions with starting
  * points of names and values subpanels.
  */
  private void syncScrollbar () {
    scrollbar.setValue(names.getStartingPoint().y);
  }

   /**
    * Attempts to draw a 3D border
   */
   private void drawBorder (Graphics g, Dimension mySize) {
     int rightx = mySize.width-2;
     int bottomy = mySize.height-2;

     g.setColor(Color.white);
     g.drawLine(1, 1, rightx, 1);
     g.drawLine(1, 1, 1, bottomy);
     g.drawLine(2, bottomy-1, rightx-1, bottomy-1);
     g.drawLine(rightx-1, 2, rightx-1, bottomy-1);

     g.setColor(Color.darkGray);
     g.drawLine(1, bottomy, rightx, bottomy);
     g.drawLine(rightx, 1, rightx, bottomy);
     g.drawLine(2, 2, rightx-1, 2);
     g.drawLine(2, 2, 2, bottomy-1);
   }

   /** 
    * Draws a 3D separator line between name and value columns.
    * Space between 3D borders is in background color
   */
   private void drawSeparator(Graphics g, int columnWidth, Dimension mySize) {
     g.setColor(Color.white);
     g.drawLine(BORDER+columnWidth, BORDER, BORDER+columnWidth,
                 mySize.height-BORDER);
     g.setColor(Color.darkGray);
     g.drawLine(BORDER+columnWidth+SEPARATOR_SIZE-1, BORDER, 
                 BORDER+columnWidth+SEPARATOR_SIZE-1, mySize.height-BORDER);
   }

   /**
    * Tries to run special dialog for editing property.
    * bug: commonProperties has to be not-null
   */
   private void runSpecialDialog (int index) {
     try {
      // calls a method that runs special input dialog,
      // setting mainmenu as its parent
       commonProps.specialPropertyInput(TopLevel.getMenu(), index);
     } catch (XFPropertyIndexOutOfBoundsException e) {
       // prints an error if property index is out of bounds
       System.err.println(e.getMessage());
       e.printStackTrace(System.err);
     } catch (XFPropertyNoSpecialInputException e) {
       // prints an error if property has no special input
       System.err.println(e.getMessage());
       e.printStackTrace(System.err);
     } catch (XFNameNotFoundException e) {
       // prints an error if bad name in layout is given
       System.err.println(e.getMessage());
       e.printStackTrace(System.err);
     }
   }

   /**
    * Private method which fills names and values containers with
    * new elements taken from commonProps object instance. 
   */
   private synchronized void fillIn (XFCommonProperties commonProperties) {
     // clear selection, if any
     boxEscaped();

     Vector indexes = commonProperties.getIndexes();
     Vector selected = commonProperties.getSelectedComponents();
     if ((selected==null) || selected.size()==0) return;
     XelfiDesignComponent first = (XelfiDesignComponent)(selected.elementAt(0));
     String[] firstPropertyNames = null;
     switch (commonProperties.getPage()) {
       case XFDesignConstants.CI_PG_PROPERTIES:
         firstPropertyNames = first.getPropertyNames();
         break;
       case XFDesignConstants.CI_PG_EVENTS:
         firstPropertyNames = first.getEventNames();
         break;
       case XFDesignConstants.CI_PG_LAYOUT:
         XelfiDesignLayout xdl =
                     ((XelfiVisualContainer)((Component)first).getParent()).
                     getDesignLayout();
         firstPropertyNames = xdl.getPropertyNames();
         break;
       // wrong page - return
       default:
         return;
     }
     // fills in names and values
     for (int i=0; i<indexes.size(); i++) {
      int  propertyIndex =
                         ((Integer)(((Vector)(indexes.elementAt(i))).
                         elementAt(0))).intValue();
       XFPropertyStringBox stringBox;
       if (propertyIndex<0) {
         // well known properties - ask commonProps object
         stringBox = new XFPropertyStringBox(i,
               commonProperties.getWellKnownPropertyName(propertyIndex));
       } else {
         // okay, we've retrieved name from getxxxxnames...
         stringBox = new XFPropertyStringBox(i,
                   firstPropertyNames[propertyIndex]);
       }
       XFPropertyValueBox valueBox = new XFPropertyValueBox(values, i,
                               commonProperties);
       names.addProperty(stringBox);
       values.addProperty(valueBox);
     }
   }
}



