/**
* Xelfi
*/

package xelfi.design.util;

import java.util.Vector;
import java.awt.*;
import xelfi.design.components.*;
import xelfi.design.forms.XelfiDesignForm;
import xelfi.util.Debug;

/**
 * XFDesignUtils class is a placeholder for different helper methods for design classes.
 * All the methods are static , so it is not necessary to make instances of this class.
 *
 * @version 	1.35, 24 Feb 1997
 * @author 	Ian Formanek, Petr Hamernik
 */

public class XFDesignUtils {
		
	/**
	* This method is intended to be started from container's paint method to do all the
	* things concerning its grid. It is not necessary to use it, but it can hide some 
	* low-level details to programmers who write their own containers and do not want to
	* go deep into implementation details. Moreover, by using this method it is sure that
	* the grid is done in a standard way.
	* @param xvc	The container which wants to have a grid painted
	* @param g		The Graphics givenas param. to paint method
	* @param gi		Xvc's gridInfo
	* @param offsX	x-offset to paint grid from
	* @param offsY	y-offset to paint grid from
	* @param imW	width of grid
	* @param imH	height of grid
	*/
	public static void paintGrid(XelfiVisualContainer xvc, Graphics g, GridInfo gi, 
			int offsX, int offsY, int imW, int imH) {
		if (!XFDesignOptions.showGrid) return;
		if (!(xvc.getDesignLayout().allowsDragging())) return;
		if (imW <= 0 || imH <=0 ) return;
		if (gi.gridX() == 1 && gi.gridY() == 1 ) return; // no grid
		if (gi.gridImage == null || gi.imWidth != imW || gi.imHeight != imH)
			new GridThread(xvc, gi, imW, imH).run();
		
		if (gi.gridImage != null)
			g.drawImage(gi.gridImage, offsX, offsY, null);
	}

	/**
	* This method is a helper utility that checks if given mouse event is the left
	* mouse event.
	* @param evt	The event we are interested in
	* @return		true if the mouse event is a left mouse event, false otherwise
	* @see @rightMouse
	*/
	public static boolean leftMouse(Event evt) {
		return	(((evt.modifiers & Event.META_MASK) == 0) &&
					 ((evt.modifiers & Event.ALT_MASK) == 0));
	}

	/**
	* This method is a helper utility that checks if given mouse event is the right
	* mouse event.
	* @param evt	The event we are interested in
	* @return		true if the mouse event is a right mouse event, false otherwise
	* @see @leftMouse
	*/
	public static boolean rightMouse(Event evt) {
		return	((evt.modifiers & Event.META_MASK) != 0);
	}
		
	/**
	* This method is a helper utility that checks if given String is java identifier
	* (starts with char [a-zA-Z_] and contains characters [a-zA-Z0-9_].
	* @param idf	The String we are interested in
	* @return		true if the String is java idf, false otherwise
	*/
	public static boolean isJavaIdf(String idf) {		
		if (idf == null || idf.length() == 0) return false;
		if (!((idf.charAt(0) >= 'a' && idf.charAt(0) <= 'z') ||
		    (idf.charAt(0) >= 'A' && idf.charAt(0) <= 'Z') ||
		     (idf.charAt(0) == '_'))) return false;
		for (int i=1; i<idf.length(); i++)
			if (!((idf.charAt(i) >= 'a' && idf.charAt(i) <= 'z') ||
			    (idf.charAt(i) >= 'A' && idf.charAt(i) <= 'Z') ||
			    (idf.charAt(i) >= '0' && idf.charAt(i) <= '9') ||
			    (idf.charAt(i) == '_'))) return false;
		return true;
	}
	
	/**
	* This method returns default name for given component's event.
	* The name is created as <name of the component>+<name of the event>.
	* Example: button1OnClick for component named button1 and OnCLick event.
	* @param xdc	the component
	* @param index	index of event
	* @return		default event name or null if some problem occured
	*/
	public static String getDefaultEventName(XelfiDesignComponent xdc, int index) {
		String names[] = xdc.getEventNames();
		int evCount = xdc.getEventCount();
		if (index<0 || index >evCount-1) return null;
		return xdc.getName()+names[index];
	}

	public static String genFont(Font ff) {
		String gen = "new Font(\"" + ff.getName() + "\", Font.";
		if (ff.isPlain())
			gen = gen + "PLAIN";
		else {
			if (ff.isBold()) {
				gen = gen + "BOLD";
				if (ff.isItalic())
					gen = gen + " + Font.ITALIC";
			}
			else if (ff.isItalic())
				gen = gen + "ITALIC";
		}
		gen = gen + ", " + ff.getSize() + ")";
		return gen;
	}

	public static String genColor(Color col) {
		String colName = xelfi.design.components.property.XColorPropertyType.getDefaultColorName(col);
		if (colName != null) return "Color."+colName;
		return "new Color(" + col.getRed() + ", " + col.getGreen() + ", " + col.getBlue() + ")";
	}
	
	public static String adjustIndent(String what, int ind) {
		return adjustIndent(what, XFDesignOptions.indent(ind));
	}
	
	public static String adjustIndent(String what, String indent) {
		if (indent == null) return what;
		String work = what;
		boolean finished = false;
		int i=0;
		int idx = -1;
		do {
			idx = work.indexOf("\n",i);
			work = work.substring(0,idx+1)+indent+work.substring(idx+1);
			i = idx+indent.length()+1;
		} while (idx !=-1);
		work = work.trim(); // remove existing whitespaces
		work = indent+work+"\n";
		return work;
	}
}