/**
* Xelfi programming environment
*/

package xelfi.editor;
/**
* One line in the document. Manages an array of character and list of positions.
*
* @version 1.0
* @author Jaroslav Tulach
*/
import java.util.StringTokenizer;
import java.util.NoSuchElementException;

import xelfi.util.BinaryTreeNode;
import xelfi.util.BinaryTreeOperation;
import xelfi.util.NodeAccessException;

final class Line extends IndexBase implements BinaryTreeNode {
  /** the text characters at this line */
  StringBuffer text;

  /** previous line */
  Line prevLine = null;

  /** next line */
  Line nextLine = null;

  /** document */
  Document doc;

  /** left son (line) */
  Line leftSon;

  /** right son (line) */
  Line rightSon;

  /** parent (line) */
  Line parent;

  /** value (count of lines in subtree) */
  int value = 1;


  /** Constructor.
  * @param sb the string buffer
  */
  Line (StringBuffer sb) {
    text = sb;
    next = null;
  }

  /** Creates new empty string buffer.
  */
  Line () {
    this (new StringBuffer ());
  }

  /** Creates line with the specified text.
  * @param s the string
  */
  Line (String s) {
    this (new StringBuffer (s));
  }

  /** Gets the line that this index belongs to
  * @return the line
  */
  final Line getLine () {
    return this;
  }

  /** Locks a index.
  * @param i the index
  */
  final synchronized void lock (Index i) {
    i.used++;
  }

  /** Unlocks a index. If the counter reaches zero the index will be removed from list.
  * @param i the index
  */
  final synchronized void unlock (Index i) {
    if (--i.used == 0) {
      // if none position uses the index and it is not "special" then
      // removes the index from list
      if ((i.prev.next = i.next) != null) {
        i.next.prev = i.prev;
        i.next.delta += i.delta;
      }
    }
  }

  /** Strips spaces from the end of the line.
  * @return the new lenght of text
  */
  final synchronized int strip () {
    int len = text.length ();
    while (len-- > 0 && text.charAt (len) == ' ');
    text.setLength (++len);
    return len;
  }

  /** Converts index to a integer.
  * @param i the index
  * @return the integer position in string
  */
  final synchronized int indexToInt (Index i) {
    int pos = 0;
    while (i != null) {
      pos += i.delta;
      i = i.getPrev ();
    }
    return pos;
  }

  /** Creates or finds index for position on specified integer.
  * This index should be imediatelly sent to constructor of Position.
  *
  * @param i the begining index
  * @param n the number of characters to right
  * @return the index on position i + n
  */
  final synchronized Index indexToRight (IndexBase i, short n) {
    while (n != 0) {
      if (i.next == null) {
        // new index at the end of line
        return new Index (i, n);
      }
      Index j = i.next;
      if (j.delta > n) {
        // we should insert new index before j and after i
        return new Index (i, n);
      }
      // move to the next item
      n -= j.delta;
      i = j;
    }

    // a index has been found
    if (i instanceof Index) {
      return (Index)i;
    } else {
      // this is not a real index
      if (i.next != null && i.next.delta == 0) {
        // there is also a index on the begin of line
        return i.next;
      } else {
        // we should create new begin of line
        return new Index (i, 0);
      }
    }
  }

  /** Creates or finds index for position on specified integer. Goes to the left.
  * This index should be imediatelly sent to constructor of Position.
  *
  * @param i the begining index
  * @param n the number of characters to left
  * @return the index on position i + n
  * @exception EndOfTextException if begining of line is reached
  */
  final synchronized Index indexToLeft (Index i, short n) throws EndOfTextException {
    while (n > 0) {
      IndexBase ib = i.prev;
      // goes to previous position
      n -= i.delta;

      if (n < 0) {
        // create new index before the previous one
        return new Index (ib, (short)-n);
      }


      if (ib instanceof Index) {
        i = (Index)ib;
      } else {
        // begin of line reached
        if (n == 0) {
          return new Index (ib, 0);
        } else {
          throw new EndOfTextException ("Begin of line reached");
        }
      }
    }
    // the one that we should find
    return i;
  }

  /** Position at the begin of the line.
  * @return the index of the position
  */
  final synchronized Index beginOfLine () {
    return indexToRight (this, (short)0);
  }

  /** Position at the end of the line.
  * @return the index of the position
  */
  final synchronized Index endOfLine () {
    return indexToRight (this, (short)text.length ());
  }

  /** Last used position or the end of the line.
  * @return the index of the position
  */
  final synchronized Index lastOfLine () {
    if (next == null) {
      return endOfLine ();
    } else {
      Index i = next;
      int d = i.delta;
      while (i.next != null) {
        i = i.next;
        d += i.delta;
      }
      if (d >= text.length ()) return i;
      else return endOfLine ();
    }
  }

  /** Is end of line. Also strips spaces from the endo fo the line.
  * @param i the index
  * @return true, if this is the last index on line
  */
  final synchronized boolean isEndOfLine (Index i) {
    return strip () <= indexToInt (i);
  }

  /** Is begin of line.
  * @param i the index
  * @return true, if this is on the begin of line
  */
  final synchronized boolean isBeginOfLine (Index i) {
    return indexToInt (i) == 0;
  }

  /** Is last of line.
  * @param i the index
  * @return true, if this is the last index on line
  */
  final synchronized boolean isLastOfLine (Index i) {
    Index j = i;
    for (;;) {
      i = i.next;
      if (i == null) return strip () <= indexToInt (j);
      if (i.delta != 0) return false;
    }
  }

  /** Finds a position on this line. Looks for position while accept method of
  * @see Acceptor returns false.
  * @param ac the acceptor to ask
  * @param indx the index to start search on
  * @param dir the direction of search (true => forward, false => backward)
  * @return the index of position that returned
  * @exception EndOfTextException if end (or begin) of line is reached
  */
  final synchronized Index find (Acceptor ac, IndexBase indx, boolean dir) throws EndOfTextException {
    return find (ac, indx, null, dir);
  }

  /** Finds a position on this line. Looks for position while accept method of
  * Acceptor returns false.
  * @param ac the acceptor to ask
  * @param indx the index to start search on
  * @param end the end index or null
  * @param dir the direction of search (true => forward, false => backward)
  * @return the index of position that returned
  * @exception EndOfTextException if end (or begin) of line is reached
  */
  final synchronized Index find (Acceptor ac, IndexBase indx, IndexBase end, boolean dir) throws EndOfTextException {
    int pos = indx instanceof Index ? indexToInt ((Index)indx) : 0;
    int len = text.length ();

    if (dir) {
      int e = end instanceof Index ? indexToInt ((Index)end) : len;
      while (pos < e && pos < len) {
        if (ac.accept (text.charAt (pos))) {
          return indexToRight (this, (short)pos);
        }
        pos++;
      }
    } else {
      if (pos >= len) pos = len - 1;
      int e = end instanceof Index ? indexToInt ((Index)end) : 0;
      while (pos-- > e) {
        if (ac.accept (text.charAt (pos))) {
          return indexToRight (this, (short)pos);
        }
      }
    }
    throw new EndOfTextException ("Nothing has been found");
  }

  /** Finds last index on the line.
  * @return the last index on line
  */
  final synchronized IndexBase lastIndex () {
    IndexBase b = this;
    while (b.next != null) {
      b = b.next;
    }
    return b;
  }


  /** Inserts new line at specified position.
  * @param i the index of the position
  * @return the following line
  */
  final synchronized Line newLine (Index i) {
    Line n;
    int pos = indexToInt (i);
    int len = text.length ();

    if (pos == 0) {
      // insert to the begin of line
      n = new Line (text);
      // creates empty buffer
      text = new StringBuffer ();
    } else {
      // insert to a middle of the line
      // moves text
      if (pos < len) {
        n = new Line (
          new StringBuffer (
            text.toString ().substring (pos, len)
          )
        );
        text.setLength (pos);
      } else {
        n = new Line ();
      }
    }

    // i is the last index
    i.prev.next = null;
    i.delta = 0;
    i.prev = n;
    n.next = i;

    // links lines
    n.prevLine = this;
    n.nextLine = nextLine;
    if (nextLine != null) {
      nextLine.prevLine = n;
    }
    nextLine = n;
    n.doc = doc;
    // link lines to their tree
    BinaryTreeOperation.insertRightSon (n, this);

    return n;
  }

  /** Inserts a text to a specified position. This text should not contain new-lines.
  * @param i the index on the line
  * @param s the string to insert
  * @return the index of beginning text
  */
  final synchronized Index insertSimple (Index i, String s) {
    int pos = indexToInt (i);

    if (text.length () <= pos) {
      int len = pos + s.length () - text.length ();
      while (len-- > 0) {
        text.append (' ');
      }
    }

    text.insert (pos, s);
    i.delta += s.length ();
    try {
      return indexToLeft (i, (short)s.length ());
    } catch (EndOfTextException ex) {
      // never raised
      return null;
    }
  }

  /** Inserts text. Can contain newlines.
  * @param i the index
  * @param s the string to insert
  * @return index on begin of inserted text
  */
  final synchronized Index insert (Index i, String s) {
    StringTokenizer st = new StringTokenizer (s, "\n", true);
    Line l = this;
    Index first = null;

    try {
      for (;;) {
        String token = st.nextToken ();
        if (token.equals ("\n")) {
          Line nl = l.newLine (i);
          if (first == null) {
            first = l.endOfLine ();
          }
          l = nl;
        } else {
          if (first == null) {
            first = l.insertSimple (i, token);
          } else {
            l.insertSimple (i, token);
          }
        }
      }
    } catch (NoSuchElementException ex) {
    }

    return first == null ? i : first;
  }

  /** Concatenates lines. This one and the next.
  * @param i the index to concate at
  */
  final synchronized void concatenateLine (Index i) {
    if (nextLine != null) {
      nextLine.concatePrevLine (this, i);
    }
  }

  /** Concatentes with previous line.
  * @param p the previous line
  * @param i the index on p to concatenate at
  */
  final private synchronized void concatePrevLine (Line p, Index i) {
    int pos = p.indexToInt (i);

    if (p.text.length () < pos) {
      int len = pos - p.text.length ();
      while (len-- > 0) {
        p.text.append (' ');
      }
    }

    p.text.append (text);

    while (i.next != null) {
      i = i.next;
      i.delta = 0;
    }

    if ((i.next = next) != null) {
      next.prev = i;
    }

    if ((p.nextLine = nextLine) != null) {
      nextLine.prevLine = p;
    }

    // deletes this line from lines' tree
    BinaryTreeOperation.deleteNode (this);

    text = null;
  }

  /** Deletes line.
  * @return previous line
  */
  final synchronized Line deleteLine () {
    Index i;
    if ((i = next) != null) for (;;) {
      i.delta = 0;
      if (i.next == null) break;
      i = i.next;
    }

    // deletes the line from their tree
    BinaryTreeOperation.deleteNode (this);

    if (nextLine != null) {
      nextLine.prevLine = prevLine;
      if (prevLine != null) {
        prevLine.nextLine = nextLine;
      }

      // prepend positions to the following line
      if (i != null) {
        nextLine.prependIndexes (next, i);
      }
      // to signal that this line is not used anymore
      text = null;

      return prevLine;
    } else {
      text = new StringBuffer ();
      return prevLine;
    }
  }

  /** Prepends some indexes.
  * @param n the first index in chain
  * @param i the last index in prepended chain
  */
  final synchronized void prependIndexes (Index n, Index i) {
    if ((i.next = next) != null) {
      next.prev = i;
    }
    next = n;
    n.prev = this;
  }

  /** Deletes characters from the line.
  * @param i the index
  * @parma n number of characters to delete
  */
  final synchronized void deleteSimple (IndexBase i, short n) {
    if (n > 0) {
      int pos = i instanceof Index ? indexToInt ((Index)i) : 0;
      int len = text.length ();

      String a = null;
      try {
        a = text.toString ().substring (0, pos);
      } catch (Exception ex1) {
        a = "";
      }
      String b = null;
      try {
        b = text.toString ().substring (pos + n, len);
      } catch (Exception ex2) {
        b = "";
      }

      text = new StringBuffer (a + b);

      Index j = i.next;
      while (j != null) {
        if (n > j.delta) {
          n -= j.delta;
          j.delta = 0;
        } else {
          j.delta -= n;
          break;
        }
        j = j.next;
      }
    }
  }

  /** Deletes the part of line between two indexes.
  * @param beg the beginning index
  * @param end the ending index
  */
  final synchronized void deleteBetween (Index beg, Index end) {
    short b = (short)indexToInt (beg);
    short e = (short)indexToInt (end);
    if (b < e) {
      deleteSimple (beg, (short)(e - b));
    }
  }

  /** Deletes rest of the line.
  * @param i the index to start from
  */
  final synchronized void deleteFrom (Index i) {
    int len = text.length ();
    int pos = indexToInt (i);
    if (pos < len) {
      deleteSimple (i, (short) (len - pos));
    }
  }

  /** Deletes begin of line. Everything till index i.
  * @param i the index to end (this index will remain)
  */
  final synchronized void deleteTill (Index i) {
    short pos = (short)indexToInt (i);
    if (pos > 0) {
      deleteSimple (this, pos);
    }
  }

  /** Deletes one character. If is positioned at the end of line then
  * concatenates two lines.
  * @param i the index to delete character at
  */
  final synchronized void deleteChar (Index i) {
    int pos = indexToInt (i);
    if (pos < strip ()) {
      deleteSimple (i, (short)1);
    } else {
      concatenateLine (i);
    }
  }

  /** Gets char at index i.
  * @param i the index on line
  * @return the character at i
  */
  final synchronized char getChar (Index i) {
    try {
      return text.charAt (indexToInt (i));
    } catch (StringIndexOutOfBoundsException ex) {
      return '\n';
    }
  }

  /** Compares two lines.
  * @param l the second line
  * @return Position.EQUAL, Position.GREATER, ...
  */
  final byte compare (Line l) {
    if (l == this) return Position.EQUAL;
    if (l.getDocument () != getDocument ()) return Position.DIFFERENT;

    BinaryTreeOperation.toRoot (this);
    try {
      if (BinaryTreeOperation.isInLeftSubTree (l, this)) {
        return Position.GREATER;
      } else {
        return Position.LESS;
      }
    } catch (NodeAccessException ex) {
      return Position.DIFFERENT;
    }
  }

  /** Returns document.
  * @return the document
  */
  final Document getDocument () {
    return doc;
  }

  /** Returns the right line number from top of document.
  * @return line number.
  */
  final int lineNumber () {
    return lineNumberStatic (this);
  }
  
  /** Return the right line number. Is synchronized to forbid multiple access
  * @param l the line to find number for
  */
  private static synchronized int lineNumberStatic (Line l) {
    BinaryTreeOperation.toRoot (l);
    l = (Line)l.getLeftSon ();
    return l == null ? 0 : l.getValue ();
  }


  /** Converts the line to string.
  * @return the string that describes whole line
  */
  public String toString () {
    strip ();
    return text.toString ();
  }

  /** Converts a part of the line to string.
  * @param from the first index
  * @param end the last one
  */
  final synchronized String toString (IndexBase from, Index end) {
    int f = from instanceof Index ? indexToInt ((Index)from) : 0;
    int e = indexToInt (end);
    int len = strip ();

    if (f > len) f = len;
    if (e > len) e = len;
    return text.toString ().substring (f, e);
  }

  /** Converts begin of line to string.
  * @param till the last index (excluded)
  */
  final synchronized String toStringTill (Index till) {
    return toString (this, till);
  }

  /** Converts end of line to string.
  * @param from the first index (included)
  */
  final synchronized String toStringFrom (Index from) {
    int f = from instanceof Index ? indexToInt ((Index)from) : 0;
    int len = strip ();

    if (f > len) f = len;
    return text.toString ().substring (f, len);
  }

  /** @return left son of this node */
  public BinaryTreeNode getLeftSon () {
    return leftSon;
  }

  /** @return right son of this node */

  public BinaryTreeNode getRightSon () {
    return rightSon;
  }

  /** @return parent this node */
  public BinaryTreeNode getParent () {
    return parent;
  }

  /** Sets new left son of this node
  * @param n the new value to set
  */
  public void setLeftSon (BinaryTreeNode n) {
    leftSon = (Line) n;
  }

  /** Sets new right son of this node
  * @param n the new value to set
  */
  public void setRightSon (BinaryTreeNode n) {
    rightSon = (Line) n;
  }

  /** Sets new parent of this node
  * @param n the new value to set
  */
  public void setParent (BinaryTreeNode n) {
    parent = (Line) n;
  }

  /** Returns the value.
  * @return the value
  */
  public int getValue () {
    return value;
  }

  /** Sets new value.
  * @param v the value
  */
  public void setValue (int v) {
    value = v;
  }
}





