/**
* Xelfi programming environment
*/

package xelfi.editor;
import java.util.NoSuchElementException;
import xelfi.util.BinaryTree;
import xelfi.util.BinaryTreeCompare;
import xelfi.util.DuplicateException;
/**
* Container for positions.
* Allows searching before and after specified one.
*
* @version 1.0
* @author Jaroslav Tulach
*/

public class PositionContainer extends Object {
  /** Compare object */
  static CompareBox comp = new CompareBox ();

  /** the tree with position box */
  BinaryTree tree = new BinaryTree (comp);

  /** Insert new position to the container.
  * @param p the position
  * @param i the info for this position
  */
  public synchronized void insert (Position p, Object i) {
    try {
      PositionBox pb = (PositionBox) tree.access (p);
      pb.info = i;
      return;
    } catch (NoSuchElementException ex) {
    }

    try {
      PositionBox box = (PositionBox) tree.getRoot ();

      PositionBox pb;
      switch (comp.compare (box, p)) {
      case BinaryTreeCompare.LESS:
        pb = box.insertAfter (p, i);
        break;
      case BinaryTreeCompare.GREATER:
      default:
        pb = box.insertBefore (p, i);
        break;
      }
      try {
        tree.insert (pb);
      } catch (DuplicateException dex) {
        // must not happen
      }
    } catch (NoSuchElementException ex) {
      // neither root is inserted
      try {
        tree.insert (new PositionBox (p, i));
      } catch (DuplicateException dex) {
      }
    }
  }

  /** Deletes position from the container.
  * @param p the position
  * @exception NoSuchElementException if the p is not in the document
  */
  public synchronized void delete (Position p) throws NoSuchElementException {
    PositionBox pb = (PositionBox) tree.access (p);
    PositionBox beg = pb.prev;
    PositionBox end = pb.next;

    if (pb == null) throw new NoSuchElementException ();

    tree.delete (pb);

    while (beg != null && beg.pos.equals (p)) {
      tree.delete (beg);
      beg = beg.prev;
    }

    while (end != null && end.pos.equals (p)) {
      tree.delete (end);
      end = end.next;
    }
    if (beg != null) beg.next = end;
    if (end != null) end.prev = beg;
  }

  /** Test if the position is in the container.
  * @param p the position
  */
  public synchronized boolean isMember (Position p) {
    return tree.isMember (p);
  }

  /** Return info for position p
  * @param p the position
  * @return the info
  * @exception NoSuchElementException if the position is not in the container
  */
  public synchronized Object getInfo (Position p) throws NoSuchElementException {
    PositionBox pb = (PositionBox) tree.access (p);
    return pb.info;
  }

  /** Sets info for position p
  * @param p the position
  * @param i the info
  * @exception NoSuchElementException if the position is not in the container
  */
  public synchronized void getInfo (Position p, Object i) throws NoSuchElementException {
    PositionBox pb = (PositionBox) tree.access (p);
    pb.info = i;
  }

  /** Previous position.
  * @param p the position
  * @return the previous position
  * @exception NoSuchElementException if the previous position does
  *   not exist
  */
  public Position previous (Position p) throws NoSuchElementException {
    return find (p, false, false);
  }

  /** Previous or the same position.
  * @param p the position
  * @return the previous position or the same if p is valid position
  * @exception NoSuchElementException if the previous position does
  *   not exist
  */
  public Position previousOrEqual (Position p) throws NoSuchElementException {
    return find (p, false, true);
  }
  
  /** Next position.
  * @param p the position
  * @return the next position
  * @exception NoSuchElementException if the next position does
  *   not exist
  */
  public Position next (Position p) throws NoSuchElementException {
    return find (p, true, false);
  }

  /** Next or the same position.
  * @param p the position
  * @return the next position or the same if p is valid position
  * @exception NoSuchElementException if the next position does
  *   not exist
  */
  public Position nextOrEqual (Position p) throws NoSuchElementException {
    return find (p, true, true);
  }

  /** Finds position.
  * @param p the position
  * @param forw search forward?
  * @param eq is position equal to p enought
  * @return the next position
  * @exception NoSuchElementException if the next position does
  *   not exist
  */
  synchronized Position find (Position p, boolean forw, boolean eq) throws NoSuchElementException {
    PositionBox pd;
    try {
      pd = (PositionBox) tree.access (p);
      if (eq) {
        // if equal position is enought
        return pd.pos;
      }
    } catch (NoSuchElementException ex) {
      // returns the closes element for p
      pd = (PositionBox) tree.getRoot ();
    }
    
    if (forw) {
      // forward
      if (pd.pos.greater (p)) {
        return pd.pos;
      }
      if (pd.next == null) throw new NoSuchElementException ();
      return pd.next.pos;
    } else {
      // backward
      if (pd.pos.less (p)) {
        return pd.pos;
      }
      if (pd.prev == null) throw new NoSuchElementException ();
      return pd.prev.pos;
    }
  }
}









