/**
* Xelfi programming environment
*/

package xelfi.templates;
import xelfi.editor.Source;
import xelfi.editor.Block;
import xelfi.editor.Position;
import xelfi.editor.EndOfTextException;
import xelfi.editor.DocumentHandler;

import java.util.NoSuchElementException;
import java.io.File;
import java.io.FileInputStream;
import java.io.DataInputStream;
import java.io.FileOutputStream;
import java.io.DataOutputStream;
import java.io.IOException;
/**
* Basic manager for plain files.
*
* @version $Id$
* @autor Jaroslav Tulach
*/
public class PlainManager extends Object implements TemplateManager, DocumentHandler {
  /** Does not store anything, because the PlainManager
  * has no special parameters.
  * @param t the template
  * @return the string to add to the specification of template
  */
  public String storeInfo (Template t) {
    return null;
  };

  /** Does not load anything.
  * @param t the template to store information to
  * @param s the string to read parameters from
  * @exception IllegalArgumentException if the parameters in s have wrong format
  */
  public void loadInfo (Template t, String s) throws IllegalArgumentException {
  };

  /** Copies the file for template t to the source.
  *
  * @param t the template for creating
  * @param s the target source
  * @exception Exception if something went wrong
  */
  public void newSource (Template t, Source s) throws Exception {
    File file = fileForTemplate (t);

    // if the class is not the basic one, then copy file
    if (t.supr != null) {
      DataInputStream is = new DataInputStream (
        new FileInputStream (file)
      );
      Block b = s.getDocument ();
      b.delete ();
      b.getBegin ().insert (is);
      is.close ();
    };

    s.setHandler (this);
  };

  /** This method reads header in the last line and tries
  * to parse the default header.
  * @param s the source
  */
  public Template identifySource (Source s) throws NoSuchElementException, IOException {
    // header is on the last non empty line
    Position p = s.getDocument ().getEnd ().beginOfLine ();
    try {
      while (p.isEndOfLine ()) {
        p = p.up ().beginOfLine ();
      };
    } catch (EndOfTextException ex) {
      // begin of text reached without finding non empty line
      throw new NoSuchElementException ();
    };

    Block b = Block.line (p);
    // if not recognized then NoSuch.. is thrown here
    Template t = lookForSignature (b.toString ());
    // delete the signature if found
    try {
      b.delete ();
    } catch (xelfi.editor.NotAllowedException ex) {
    };

    s.setHandler (this);

    return t;
  };

  /** The contents of source s is copied to the
  * template directory.
  *
  * @param old the old template
  * @param s the source
  * @param theNew the new template that should be created
  * @return returns template manager for this template (can be this or something else)
  * @exception Exception if the creating is not possible
  */
  public TemplateManager createTemplate (Template old, Source s, Template theNew) throws Exception {
    File file = fileForTemplate (theNew);
    DataOutputStream os = new DataOutputStream (
      new FileOutputStream (file)
    );
    s.getDocument ().save (os);
    os.close ();
    return this;
  };

  /** Returns file for the template t. Usually templates should be
  * stored under one special directory in class path.
  * @param t the template
  * @param ext the extension the file should have (with beginning dot)
  * @return the file for this template
  */
  public static File fileForTemplate (Template t, String ext) {
    String path = xelfi.util.XelfiPath.get ();
    char sep = java.io.File.separatorChar;
    File file = new File (
      path + sep + "templates" + sep + "patterns" + sep + t.name + ext
    );
    new File (file.getParent ()).mkdirs ();
    return file;
  };

  /** Returns file for the template t. Usually templates should be
  * stored under one special directory in class path.
  * Extension ".template" is added to the file.
  * @param t the template
  * @return the file for this template
  */
  public static File fileForTemplate (Template t) {
    return fileForTemplate (t, ".template");
  };

  private static final String signatureHeader = "// Generated from plain template \'";
  private static final String signatureEnd = "\'";

  /** Searches for right signature in the first line of document.
  * @param s the string to search in
  * @return the template that created this source
  * @exception NoSuchElementException if the template in the header is not found
  *   or there is no header
  */
  static Template lookForSignature (String s) {
    return lookForSignature (s, signatureHeader, signatureEnd);
  };

  /** Searches for right signature in the first line of document.
  * @param s the string to search in
  * @param before the begin of signature
  * @param the end of signature
  * @return the template that created this source
  * @exception NoSuchElementException if the template in the header is not found
  *   or there is no header
  */
  public static Template lookForSignature (String s, String before, String end) throws
  NoSuchElementException {
    int pos = s.indexOf (before);
    if (pos >= 0) {
      pos += before.length ();
      int e = s.indexOf (end, pos);
      if (e >= pos) {
        String name = s.substring (pos, e);
        return Template.find (name);
      };
    };
    throw new NoSuchElementException ("String \"" + s + "\" does not contains header");
  };

  /** Returns the signature for this template. This signature is in reverse
  * recognized by lookForSignature method.
  * @param t the template
  * @return the string with signature
  */
  static String createSignature (Template t) {
    return createSignature (t, signatureHeader, signatureEnd);
  };

  /** Returns the signature for this template. This signature is in reverse
  * recognized by lookForSignature method.
  * @param t the template
  * @param before the begin of signature
  * @param the end of signature
  * @return the string with signature
  */
  public static String createSignature (Template t, String before, String end) {
    return before + t.name + end;
  };

  // methods of DocumentHandler

  /** Calls StandardSaver.save () to save the source file.
  * @see xelfi.editor.StandardSaver#save
  * @param s the source
  * @param backup is true if the backup files should be created
  * @exception IOException if an error occures and saving is not possible
  */
  public void saveSource (Source s, boolean backup) throws IOException {
    // template for source s
    Template t = Template.identifySource (s);

    if (backup) {
      try {
        xelfi.editor.StandardDocumentHandler.backup (s.getSource ());
      } catch (IOException ex) {
      };
    };
    DataOutputStream os = new DataOutputStream (new FileOutputStream (s.getSource ()));
    xelfi.editor.StandardDocumentHandler.save (s, os);
    if (t != Template.basic) {
      // if the template is not the basic one then
      // add signature to the end of source
      os.writeBytes (createSignature (t));
      os.writeByte ('\n');
    };
    os.close ();
  };
  
  /** Shows the editor. Now calls top level. Bleee....
  * @param s the source
  */
  public void showAll (Source s) {
    xelfi.top.TopLevel.showEditor (s, true);
  }

};




